@@ -13,65 +13,63 @@
 """Copy interface
 """
 
-from cgi import escape
-from marshal import dumps
-from marshal import loads
 import logging
 import re
-import sys
 import tempfile
-from urllib import quote
-from urllib import unquote
 import warnings
+from json import dumps
+from json import loads
 from zlib import compress
 from zlib import decompressobj
-import transaction
 
+import six
+from six.moves.urllib.parse import quote
+from six.moves.urllib.parse import unquote
+
+import transaction
 from AccessControl import ClassSecurityInfo
 from AccessControl import getSecurityManager
 from AccessControl.class_init import InitializeClass
-from AccessControl.Permissions import view_management_screens
 from AccessControl.Permissions import copy_or_move
 from AccessControl.Permissions import delete_objects
+from AccessControl.Permissions import view_management_screens
 from Acquisition import aq_base
 from Acquisition import aq_inner
 from Acquisition import aq_parent
-from App.Dialogs import MessageDialog
-from App.special_dtml import HTML
 from App.special_dtml import DTMLFile
 from ExtensionClass import Base
-from webdav.Lockable import ResourceLockedError
-from zExceptions import Unauthorized, BadRequest
-from ZODB.POSException import ConflictError
-from zope.interface import implements
-from zope.event import notify
-from zope.lifecycleevent import ObjectCopiedEvent
-from zope.lifecycleevent import ObjectMovedEvent
-from zope.container.contained import notifyContainerModified
-
-from OFS.event import ObjectWillBeMovedEvent
 from OFS.event import ObjectClonedEvent
+from OFS.event import ObjectWillBeMovedEvent
 from OFS.interfaces import ICopyContainer
 from OFS.interfaces import ICopySource
-from OFS.Moniker import loadMoniker
 from OFS.Moniker import Moniker
+from OFS.Moniker import loadMoniker
 from OFS.subscribers import compatibilityCall
+from zExceptions import BadRequest
+from zExceptions import ResourceLockedError
+from zExceptions import Unauthorized
+from ZODB.POSException import ConflictError
+from zope.container.contained import notifyContainerModified
+from zope.event import notify
+from zope.interface import implementer
+from zope.lifecycleevent import ObjectCopiedEvent
+from zope.lifecycleevent import ObjectMovedEvent
 
 
 class CopyError(Exception):
     pass
 
+
 copy_re = re.compile('^copy([0-9]*)_of_(.*)')
 logger = logging.getLogger('OFS')
 _marker = []
 
 
+@implementer(ICopyContainer)
 class CopyContainer(Base):
 
     """Interface for containerish objects which allow cut/copy/paste"""
 
-    implements(ICopyContainer)
-
     security = ClassSecurityInfo()
 
     # The following three methods should be overridden to store sub-objects
@@ -95,58 +93,57 @@ class CopyContainer(Base):
     def manage_CopyContainerAllItems(self, REQUEST):
         return [self._getOb(i) for i in REQUEST['ids']]
 
-    security.declareProtected(delete_objects, 'manage_cutObjects')
+    @security.protected(delete_objects)
     def manage_cutObjects(self, ids=None, REQUEST=None):
         """Put a reference to the objects named in ids in the clip board"""
         if ids is None and REQUEST is not None:
-            return eNoItemsSpecified
+            raise BadRequest('No items specified')
         elif ids is None:
             raise ValueError('ids must be specified')
 
-        if type(ids) is str:
-            ids=[ids]
-        oblist=[]
+        if isinstance(ids, str):
+            ids = [ids]
+        oblist = []
         for id in ids:
-            ob=self._getOb(id)
+            ob = self._getOb(id)
 
             if ob.wl_isLocked():
-                raise ResourceLockedError('Object "%s" is locked via WebDAV'
-                                            % ob.getId())
+                raise ResourceLockedError('Object "%s" is locked' % ob.getId())
 
             if not ob.cb_isMoveable():
-                raise CopyError(eNotSupported % escape(id))
+                raise CopyError('Not Supported')
             m = Moniker(ob)
             oblist.append(m.dump())
-        cp=(1, oblist)
-        cp=_cb_encode(cp)
+        cp = (1, oblist)
+        cp = _cb_encode(cp)
         if REQUEST is not None:
-            resp=REQUEST['RESPONSE']
+            resp = REQUEST['RESPONSE']
             resp.setCookie('__cp', cp, path='%s' % cookie_path(REQUEST))
             REQUEST['__cp'] = cp
             return self.manage_main(self, REQUEST)
         return cp
 
-    security.declareProtected(view_management_screens, 'manage_copyObjects')
+    @security.protected(view_management_screens)
     def manage_copyObjects(self, ids=None, REQUEST=None, RESPONSE=None):
         """Put a reference to the objects named in ids in the clip board"""
         if ids is None and REQUEST is not None:
-            return eNoItemsSpecified
+            raise BadRequest('No items specified')
         elif ids is None:
             raise ValueError('ids must be specified')
 
-        if type(ids) is str:
-            ids=[ids]
-        oblist=[]
+        if isinstance(ids, str):
+            ids = [ids]
+        oblist = []
         for id in ids:
-            ob=self._getOb(id)
+            ob = self._getOb(id)
             if not ob.cb_isCopyable():
-                raise CopyError(eNotSupported % escape(id))
+                raise CopyError('Not Supported')
             m = Moniker(ob)
             oblist.append(m.dump())
-        cp=(0, oblist)
-        cp=_cb_encode(cp)
+        cp = (0, oblist)
+        cp = _cb_encode(cp)
         if REQUEST is not None:
-            resp=REQUEST['RESPONSE']
+            resp = REQUEST['RESPONSE']
             resp.setCookie('__cp', cp, path='%s' % cookie_path(REQUEST))
             REQUEST['__cp'] = cp
             return self.manage_main(self, REQUEST)
@@ -166,8 +163,8 @@ class CopyContainer(Base):
         while 1:
             if self._getOb(id, None) is None:
                 return id
-            id='copy%s_of_%s' % (n and n+1 or '', orig_id)
-            n=n+1
+            id = 'copy%s_of_%s' % (n and n + 1 or '', orig_id)
+            n = n + 1
 
     def _pasteObjects(self, cp, cb_maxsize=0):
         """Paste previously copied objects into the current object.
@@ -188,12 +185,12 @@ class CopyContainer(Base):
         Where `operator` is 0 for a copy operation and 1 for a move operation.
         """
         if cp is None:
-            raise CopyError(eNoData)
+            raise CopyError('No clipboard data found.')
 
         try:
             op, mdatas = _cb_decode(cp, cb_maxsize)
-        except:
-            raise CopyError(eInvalid)
+        except Exception as e:
+            six.raise_from(CopyError('Clipboard Error'), e)
 
         oblist = []
         app = self.getPhysicalRoot()
@@ -203,9 +200,9 @@ class CopyContainer(Base):
                 ob = m.bind(app)
             except ConflictError:
                 raise
-            except:
-                raise CopyError(eNotFound)
-            self._verifyObjectPaste(ob, validate_src=op+1)
+            except Exception:
+                raise CopyError('Item Not Found')
+            self._verifyObjectPaste(ob, validate_src=op + 1)
             oblist.append(ob)
 
         result = []
@@ -214,17 +211,14 @@ class CopyContainer(Base):
             for ob in oblist:
                 orig_id = ob.getId()
                 if not ob.cb_isCopyable():
-                    raise CopyError(eNotSupported % escape(orig_id))
+                    raise CopyError('Not Supported')
 
                 try:
                     ob._notifyOfCopyTo(self, op=0)
                 except ConflictError:
                     raise
-                except:
-                    raise CopyError(MessageDialog(
-                        title="Copy Error",
-                        message=sys.exc_info()[1],
-                        action='manage_main'))
+                except Exception:
+                    raise CopyError('Copy Error')
 
                 id = self._get_id(orig_id)
                 result.append({'id': orig_id, 'new_id': id})
@@ -249,21 +243,17 @@ class CopyContainer(Base):
             for ob in oblist:
                 orig_id = ob.getId()
                 if not ob.cb_isMoveable():
-                    raise CopyError(eNotSupported % escape(orig_id))
+                    raise CopyError('Not Supported')
 
                 try:
                     ob._notifyOfCopyTo(self, op=1)
                 except ConflictError:
                     raise
-                except:
-                    raise CopyError(MessageDialog(
-                        title="Move Error",
-                        message=sys.exc_info()[1],
-                        action='manage_main'))
+                except Exception:
+                    raise CopyError('Move Error')
 
                 if not sanity_check(self, ob):
-                    raise CopyError(
-                            "This object cannot be pasted into itself")
+                    raise CopyError("This object cannot be pasted into itself")
 
                 orig_container = aq_parent(aq_inner(ob))
                 if aq_base(orig_container) is aq_base(self):
@@ -310,7 +300,7 @@ class CopyContainer(Base):
 
         return op, result
 
-    security.declareProtected(view_management_screens, 'manage_pasteObjects')
+    @security.protected(view_management_screens)
     def manage_pasteObjects(self, cb_copy_data=None, REQUEST=None):
         """Paste previously copied objects into the current object.
 
@@ -325,9 +315,7 @@ class CopyContainer(Base):
         list of dicts {'id': orig_id, 'new_id': new_id} otherwise it renders
         a HTML page.
         """
-        if (cb_copy_data is None and
-                REQUEST is not None and
-                REQUEST.has_key('__cp')):
+        if cb_copy_data is None and REQUEST is not None and '__cp' in REQUEST:
             cb_copy_data = REQUEST['__cp']
         op, result = self._pasteObjects(cb_copy_data, cb_maxsize=8192)
 
@@ -335,20 +323,20 @@ class CopyContainer(Base):
             if op == 0:
                 cb_valid = 1
             elif op == 1:
-                REQUEST['RESPONSE'].setCookie('__cp', 'deleted',
-                                    path='%s' % cookie_path(REQUEST),
-                                    expires='Wed, 31-Dec-97 23:59:59 GMT')
+                REQUEST['RESPONSE'].setCookie(
+                    '__cp', 'deleted',
+                    path='%s' % cookie_path(REQUEST),
+                    expires='Wed, 31-Dec-97 23:59:59 GMT')
                 REQUEST['__cp'] = None
                 cb_valid = 0
-            return self.manage_main(self, REQUEST, update_menu=1,
-                                    cb_dataValid=cb_valid)
+            return self.manage_main(self, REQUEST, cb_dataValid=cb_valid)
 
         return result
 
-    security.declareProtected(view_management_screens, 'manage_renameForm')
+    security.declareProtected(view_management_screens, 'manage_renameForm')  # NOQA: D001,E501
     manage_renameForm = DTMLFile('dtml/renameForm', globals())
 
-    security.declareProtected(view_management_screens, 'manage_renameObjects')
+    @security.protected(view_management_screens)
     def manage_renameObjects(self, ids=[], new_ids=[], REQUEST=None):
         """Rename several sub-objects"""
         if len(ids) != len(new_ids):
@@ -357,39 +345,31 @@ class CopyContainer(Base):
             if ids[i] != new_ids[i]:
                 self.manage_renameObject(ids[i], new_ids[i], REQUEST)
         if REQUEST is not None:
-            return self.manage_main(self, REQUEST, update_menu=1)
-        return None
+            return self.manage_main(self, REQUEST)
 
-    security.declareProtected(view_management_screens, 'manage_renameObject')
+    @security.protected(view_management_screens)
     def manage_renameObject(self, id, new_id, REQUEST=None):
         """Rename a particular sub-object.
         """
         try:
             self._checkId(new_id)
-        except:
-            raise CopyError(MessageDialog(
-                title='Invalid Id',
-                message=sys.exc_info()[1],
-                action ='manage_main'))
+        except Exception:
+            raise CopyError('Invalid Id')
 
         ob = self._getOb(id)
 
         if ob.wl_isLocked():
-            raise ResourceLockedError('Object "%s" is locked via WebDAV'
-                                        % ob.getId())
+            raise ResourceLockedError('Object "%s" is locked' % ob.getId())
         if not ob.cb_isMoveable():
-            raise CopyError(eNotSupported % escape(id))
+            raise CopyError('Not Supported')
         self._verifyObjectPaste(ob)
 
         try:
             ob._notifyOfCopyTo(self, op=1)
         except ConflictError:
             raise
-        except:
-            raise CopyError(MessageDialog(
-                title="Rename Error",
-                message=sys.exc_info()[1],
-                action ='manage_main'))
+        except Exception:
+            raise CopyError('Rename Error')
 
         notify(ObjectWillBeMovedEvent(ob, self, id, self, new_id))
 
@@ -420,27 +400,18 @@ class CopyContainer(Base):
         ob._postCopy(self, op=1)
 
         if REQUEST is not None:
-            return self.manage_main(self, REQUEST, update_menu=1)
-        return None
+            return self.manage_main(self, REQUEST)
 
-    # Why did we give this a manage_ prefix if its really
-    # supposed to be public since it does its own auth ?
-    #
-    # Because it's still a "management" function.
-
-    security.declarePublic('manage_clone')
+    @security.public
     def manage_clone(self, ob, id, REQUEST=None):
         """Clone an object, creating a new object with the given id.
         """
         if not ob.cb_isCopyable():
-            raise CopyError(eNotSupported % escape(ob.getId()))
+            raise CopyError('Not Supported')
         try:
             self._checkId(id)
-        except:
-            raise CopyError(MessageDialog(
-                title='Invalid Id',
-                message=sys.exc_info()[1],
-                action ='manage_main'))
+        except Exception:
+            raise CopyError('Invalid Id')
 
         self._verifyObjectPaste(ob)
 
@@ -448,11 +419,8 @@ class CopyContainer(Base):
             ob._notifyOfCopyTo(self, op=0)
         except ConflictError:
             raise
-        except:
-            raise CopyError(MessageDialog(
-                title="Clone Error",
-                message=sys.exc_info()[1],
-                action='manage_main'))
+        except Exception:
+            raise CopyError('Clone Error')
 
         orig_ob = ob
         ob = ob._getCopy(self)
@@ -472,15 +440,19 @@ class CopyContainer(Base):
 
     def cb_dataValid(self):
         # Return true if clipboard data seems valid.
-        try:    cp=_cb_decode(self.REQUEST['__cp'])
-        except: return 0
+        try:
+            _cb_decode(self.REQUEST['__cp'])
+        except Exception:
+            return 0
         return 1
 
     def cb_dataItems(self):
         # List of objects in the clip board
-        try:    cp=_cb_decode(self.REQUEST['__cp'])
-        except: return []
-        oblist=[]
+        try:
+            cp = _cb_decode(self.REQUEST['__cp'])
+        except Exception:
+            return []
+        oblist = []
 
         app = self.getPhysicalRoot()
         for mdata in cp[1]:
@@ -488,7 +460,7 @@ class CopyContainer(Base):
             oblist.append(m.bind(app))
         return oblist
 
-    validClipData=cb_dataValid
+    validClipData = cb_dataValid
 
     def _verifyObjectPaste(self, object, validate_src=1):
         # Verify whether the current user is allowed to paste the
@@ -506,25 +478,16 @@ class CopyContainer(Base):
         # heirarchy).
 
         if not hasattr(object, 'meta_type'):
-            raise CopyError(MessageDialog(
-                  title   = 'Not Supported',
-                  message = ('The object <em>%s</em> does not support this' \
-                             ' operation' % escape(absattr(object.id))),
-                  action  = 'manage_main'))
+            raise CopyError('Not Supported')
 
         if not hasattr(self, 'all_meta_types'):
-            raise CopyError(MessageDialog(
-                  title   = 'Not Supported',
-                  message = 'Cannot paste into this object.',
-                  action  = 'manage_main'))
+            raise CopyError('Cannot paste into this object.')
 
-        method_name = None
         mt_permission = None
         meta_types = absattr(self.all_meta_types)
 
         for d in meta_types:
             if d['name'] == object.meta_type:
-                method_name = d['action']
                 mt_permission = d.get('permission')
                 break
 
@@ -537,39 +500,29 @@ class CopyContainer(Base):
                     # clipboard.
                     try:
                         parent = aq_parent(aq_inner(object))
-                    except:
+                    except Exception:
                         parent = None
 
                     if not sm.validate(None, parent, None, object):
                         raise Unauthorized(absattr(object.id))
 
-                    if validate_src == 2: # moving
+                    if validate_src == 2:  # moving
                         if not sm.checkPermission(delete_objects, parent):
                             raise Unauthorized('Delete not allowed.')
             else:
-                raise CopyError(MessageDialog(
-                    title = 'Insufficient Privileges',
-                    message = ('You do not possess the %s permission in the '
-                               'context of the container into which you are '
-                               'pasting, thus you are not able to perform '
-                               'this operation.' % mt_permission),
-                    action = 'manage_main'))
+                raise CopyError('Insufficient privileges')
         else:
-            raise CopyError(MessageDialog(
-                title = 'Not Supported',
-                message = ('The object <em>%s</em> does not support this '
-                           'operation.' % escape(absattr(object.id))),
-                action = 'manage_main'))
+            raise CopyError('Not Supported')
+
 
 InitializeClass(CopyContainer)
 
 
+@implementer(ICopySource)
 class CopySource(Base):
 
     """Interface for objects which allow themselves to be copied."""
 
-    implements(ICopySource)
-
     # declare a dummy permission for Copy or Move here that we check
     # in cb_isCopyable.
     security = ClassSecurityInfo()
@@ -599,19 +552,17 @@ class CopySource(Base):
 
         if self._p_jar is None:
             raise CopyError(
-                'Object "%s" needs to be in the database to be copied' %
-                `self`)
+                'Object "%r" needs to be in the database to be copied' % self)
         if container._p_jar is None:
             raise CopyError(
-                'Container "%s" needs to be in the database' %
-                `container`)
+                'Container "%r" needs to be in the database' % container)
 
         # Ask an object for a new copy of itself.
-        f=tempfile.TemporaryFile()
-        self._p_jar.exportFile(self._p_oid,f)
-        f.seek(0)
-        ob=container._p_jar.importFile(f)
-        f.close()
+        with tempfile.TemporaryFile() as f:
+            self._p_jar.exportFile(self._p_oid, f)
+            f.seek(0)
+            ob = container._p_jar.importFile(f)
+
         # Cleanup the copy.  It may contain private objects that the current
         # user is not allowed to see.
         sm = getSecurityManager()
@@ -632,7 +583,7 @@ class CopySource(Base):
                     # We do not use cp._delObject, because this would fire
                     # events that are needless for objects that are not even in
                     # an Acquisition chain yet.
-                    logger.warn(
+                    logger.warning(
                         'While copying %s to %s, removed %s from copy '
                         'because user is not allowed to view the original.',
                         '/'.join(self.getPhysicalPath()),
@@ -656,7 +607,7 @@ class CopySource(Base):
 
     def _setId(self, id):
         # Called to set the new id of a copied object.
-        self.id=id
+        self.id = id
 
     def cb_isCopyable(self):
         # Is object copyable? Returns 0 or 1
@@ -672,8 +623,10 @@ class CopySource(Base):
             return 0
         if hasattr(self, '_p_jar') and self._p_jar is None:
             return 0
-        try:    n=aq_parent(aq_inner(self))._reserved_names
-        except: n=()
+        try:
+            n = aq_parent(aq_inner(self))._reserved_names
+        except Exception:
+            n = ()
         if absattr(self.id) in n:
             return 0
         if not self.cb_userHasCopyOrMovePermission():
@@ -684,6 +637,7 @@ class CopySource(Base):
         if getSecurityManager().checkPermission(copy_or_move, self):
             return 1
 
+
 InitializeClass(CopySource)
 
 
@@ -700,12 +654,28 @@ def sanity_check(c, ob):
             return 1
         c = aq_parent(inner)
 
+
 def absattr(attr):
-    if callable(attr): return attr()
+    if callable(attr):
+        return attr()
     return attr
 
+
 def _cb_encode(d):
-    return quote(compress(dumps(d), 9))
+    """Encode a list of IDs for storage in a cookie.
+
+    ``d`` must be a list or tuple of text IDs.
+
+    Return text.
+    """
+    json_bytes = dumps(d).encode('utf-8')
+    squashed_bytes = compress(json_bytes, 2)  # -> bytes w/ useful encoding
+    # quote for embeding in cookie
+    if six.PY2:
+        return quote(squashed_bytes)
+    else:
+        return quote(squashed_bytes.decode('latin-1'))
+
 
 def _cb_decode(s, maxsize=8192):
     """Decode a list of IDs from storage in a cookie.
@@ -716,88 +686,18 @@ def _cb_decode(s, maxsize=8192):
     Return a list of text IDs.
     """
     dec = decompressobj()
-    data = dec.decompress(unquote(s), maxsize)
+    if six.PY2:
+        squashed = unquote(s)
+    else:
+        squashed = unquote(s).encode('latin-1')
+    data = dec.decompress(squashed, maxsize)
     if dec.unconsumed_tail:
         raise ValueError
-    return loads(data)
+    json_bytes = data.decode('utf-8')
+    return loads(json_bytes)
+
 
 def cookie_path(request):
     # Return a "path" value for use in a cookie that refers
     # to the root of the Zope object space.
     return request['BASEPATH1'] or "/"
-
-
-
-fMessageDialog = HTML("""
-<HTML>
-<HEAD>
-<TITLE>&dtml-title;</TITLE>
-</HEAD>
-<BODY BGCOLOR="#FFFFFF">
-<FORM ACTION="&dtml-action;" METHOD="GET" <dtml-if
- target>TARGET="&dtml-target;"</dtml-if>>
-<TABLE BORDER="0" WIDTH="100%%" CELLPADDING="10">
-<TR>
-  <TD VALIGN="TOP">
-  <BR>
-  <CENTER><B><FONT SIZE="+6" COLOR="#77003B">!</FONT></B></CENTER>
-  </TD>
-  <TD VALIGN="TOP">
-  <BR><BR>
-  <CENTER>
-  <dtml-var message>
-  </CENTER>
-  </TD>
-</TR>
-<TR>
-  <TD VALIGN="TOP">
-  </TD>
-  <TD VALIGN="TOP">
-  <CENTER>
-  <INPUT TYPE="SUBMIT" VALUE="   Ok   ">
-  </CENTER>
-  </TD>
-</TR>
-</TABLE>
-</FORM>
-</BODY></HTML>""", target='', action='manage_main', title='Changed')
-
-
-eNoData=MessageDialog(
-        title='No Data',
-        message='No clipboard data found.',
-        action ='manage_main',)
-
-eInvalid=MessageDialog(
-         title='Clipboard Error',
-         message='The data in the clipboard could not be read, possibly due ' \
-         'to cookie data being truncated by your web browser. Try copying ' \
-         'fewer objects.',
-         action ='manage_main',)
-
-eNotFound=MessageDialog(
-          title='Item Not Found',
-          message='One or more items referred to in the clipboard data was ' \
-          'not found. The item may have been moved or deleted after you ' \
-          'copied it.',
-          action ='manage_main',)
-
-eNotSupported=fMessageDialog(
-              title='Not Supported',
-              message=(
-              'The action against the <em>%s</em> object could not be carried '
-              'out. '
-              'One of the following constraints caused the problem: <br><br>'
-              'The object does not support this operation.'
-              '<br><br>-- OR --<br><br>'
-              'The currently logged-in user does not have the <b>Copy or '
-              'Move</b> permission respective to the object.'
-              ),
-              action ='manage_main',)
-
-eNoItemsSpecified=MessageDialog(
-                  title='No items specified',
-                  message='You must select one or more items to perform ' \
-                  'this operation.',
-                  action ='manage_main'
-                  )
