@@ -12,85 +12,102 @@
 ##############################################################################
 """DTML Method objects.
 """
-from urllib import quote
+import re
+
+from six import PY2
+from six import PY3
+from six import binary_type
+from six import text_type
+from six.moves.urllib.parse import quote
 
-from AccessControl.class_init import InitializeClass
-from AccessControl.SecurityInfo import ClassSecurityInfo
-from Acquisition import Implicit
-from App.special_dtml import DTMLFile
-from App.special_dtml import HTML
-from DateTime.DateTime import DateTime
 from AccessControl import getSecurityManager
-from AccessControl.Permissions import view_management_screens
-from AccessControl.Permissions import change_proxy_roles
-from AccessControl.Permissions import view as View
+from AccessControl.class_init import InitializeClass
+from AccessControl.Permissions import change_proxy_roles  # NOQA
 from AccessControl.Permissions import ftp_access
+from AccessControl.Permissions import view as View
+from AccessControl.Permissions import view_management_screens
 from AccessControl.requestmethod import requestmethod
+from AccessControl.SecurityInfo import ClassSecurityInfo
 from AccessControl.tainted import TaintedString
+from Acquisition import Implicit
+from App.special_dtml import HTML
+from App.special_dtml import DTMLFile
+from DocumentTemplate.DT_Util import ParseError
 from DocumentTemplate.permissions import change_dtml_methods
 from DocumentTemplate.security import RestrictedDTML
+from OFS import bbb
 from OFS.Cache import Cacheable
-from OFS.History import Historical
-from OFS.History import html_diff
 from OFS.role import RoleManager
 from OFS.SimpleItem import Item_w__name__
-from OFS.ZDOM import ElementWithTitle
-from webdav.Lockable import ResourceLockedError
+from OFS.SimpleItem import PathReprProvider
 from zExceptions import Forbidden
+from zExceptions import ResourceLockedError
 from zExceptions.TracebackSupplement import PathTracebackSupplement
-from ZPublisher.Iterators import IStreamIterator
 from zope.contenttype import guess_content_type
+from ZPublisher.HTTPRequest import default_encoding
+from ZPublisher.Iterators import IStreamIterator
 
 
 _marker = []  # Create a new marker object.
 
-class DTMLMethod(RestrictedDTML,
-                 HTML,
-                 Implicit,
-                 RoleManager,
-                 ElementWithTitle,
-                 Item_w__name__,
-                 Historical,
-                 Cacheable,
-                ):
+
+class Code(object):
+    # Documents masquerade as functions:
+    pass
+
+
+class DTMLMethod(
+    PathReprProvider,
+    RestrictedDTML,
+    HTML,
+    Implicit,
+    RoleManager,
+    Item_w__name__,
+    Cacheable
+):
     """ DocumentTemplate.HTML objects that act as methods of their containers.
     """
     meta_type = 'DTML Method'
+    zmi_icon = 'far fa-file-alt'
     _proxy_roles = ()
-    index_html = None # Prevent accidental acquisition
+    index_html = None  # Prevent accidental acquisition
     _cache_namespace_keys = ()
+    _locked_error_text = 'This DTML Method is locked.'
 
     security = ClassSecurityInfo()
     security.declareObjectProtected(View)
 
-    # Documents masquerade as functions:
-    class func_code:
-        pass
-    func_code = func_code()
-    func_code.co_varnames = 'self', 'REQUEST', 'RESPONSE'
-    func_code.co_argcount = 3
+    __code__ = Code()
+    __code__.co_varnames = 'self', 'REQUEST', 'RESPONSE'
+    __code__.co_argcount = 3
+    __defaults__ = None
 
     manage_options = ((
-        {'label': 'Edit', 'action': 'manage_main'},
-        {'label': 'View', 'action': ''},
-        {'label': 'Proxy', 'action': 'manage_proxyForm'},
-        )
-        + Historical.manage_options
-        + RoleManager.manage_options
-        + Item_w__name__.manage_options
-        + Cacheable.manage_options
-        )
-
-    # Careful in permission changes--used by DTMLDocument!
-
-    security.declareProtected(change_dtml_methods, 'manage_historyCopy')
-    security.declareProtected(change_dtml_methods, 'manage_beforeHistoryCopy')
-    security.declareProtected(change_dtml_methods, 'manage_afterHistoryCopy')
+        {
+            'label': 'Edit',
+            'action': 'manage_main',
+        },
+        {
+            'label': 'View',
+            'action': '',
+        },
+        {
+            'label': 'Proxy',
+            'action': 'manage_proxyForm',
+        },
+    ) + RoleManager.manage_options
+      + Item_w__name__.manage_options
+      + Cacheable.manage_options
+    )
 
     # More reasonable default for content-type for http HEAD requests.
     default_content_type = 'text/html'
 
-    security.declareProtected(View, '__call__')
+    def errQuote(self, s):
+        # Quoting is done when rendering the error in the template.
+        return s
+
+    @security.protected(View)
     def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
         """Render using the given client object
 
@@ -103,17 +120,19 @@ class DTMLMethod(RestrictedDTML,
         if not self._cache_namespace_keys:
             data = self.ZCacheable_get(default=_marker)
             if data is not _marker:
-                if ( IStreamIterator.isImplementedBy(data) and
-                     RESPONSE is not None ):
+                if IStreamIterator.isImplementedBy(data) and \
+                   RESPONSE is not None:
                     # This is a stream iterator and we need to set some
                     # headers now before giving it to medusa
-                    if RESPONSE.headers.get('content-length', None) is None:
+                    headers_get = RESPONSE.headers.get
+
+                    if headers_get('content-length', None) is None:
                         RESPONSE.setHeader('content-length', len(data))
 
-                    if ( RESPONSE.headers.get('content-type', None) is None and
-                         RESPONSE.headers.get('Content-type', None) is None ):
-                        ct = ( self.__dict__.get('content_type') or
-                               self.default_content_type )
+                    if headers_get('content-type', None) is None and \
+                       headers_get('Content-type', None) is None:
+                        ct = (self.__dict__.get('content_type')
+                              or self.default_content_type)
                         RESPONSE.setHeader('content-type', ct)
 
                 # Return cached results.
@@ -125,7 +144,7 @@ class DTMLMethod(RestrictedDTML,
 
         security = getSecurityManager()
         security.addContext(self)
-        if self.__dict__.has_key('validate'):
+        if 'validate' in self.__dict__:
             first_time_through = 0
         else:
             self.__dict__['validate'] = security.DTMLValidate
@@ -134,7 +153,7 @@ class DTMLMethod(RestrictedDTML,
 
             if client is None:
                 # Called as subtemplate, so don't need error propagation!
-                r = apply(HTML.__call__, (self, client, REQUEST), kw)
+                r = HTML.__call__(self, client, REQUEST, **kw)
                 if RESPONSE is None:
                     result = r
                 else:
@@ -143,8 +162,9 @@ class DTMLMethod(RestrictedDTML,
                     self.ZCacheable_set(result)
                 return result
 
-            r = apply(HTML.__call__, (self, client, REQUEST), kw)
-            if type(r) is not str or RESPONSE is None:
+            r = HTML.__call__(self, client, REQUEST, **kw)
+
+            if RESPONSE is None or not isinstance(r, str):
                 if not self._cache_namespace_keys:
                     self.ZCacheable_set(r)
                 return r
@@ -154,12 +174,16 @@ class DTMLMethod(RestrictedDTML,
             if first_time_through:
                 del self.__dict__['validate']
 
-        have_key = RESPONSE.headers.has_key
+        have_key = RESPONSE.headers.__contains__
         if not (have_key('content-type') or have_key('Content-Type')):
-            if self.__dict__.has_key('content_type'):
+            if 'content_type' in self.__dict__:
                 c = self.content_type
             else:
-                c, e = guess_content_type(self.getId(), r)
+                encoding = getattr(self, 'encoding', default_encoding)
+                if PY2 and not isinstance(r, text_type):
+                    # Prevent double-encoding edge cases under Python 2
+                    r = r.decode(encoding)
+                c, e = guess_content_type(self.getId(), r.encode(encoding))
             RESPONSE.setHeader('Content-Type', c)
         result = decapitate(r, RESPONSE)
         if not self._cache_namespace_keys:
@@ -178,7 +202,7 @@ class DTMLMethod(RestrictedDTML,
             for key in self._cache_namespace_keys:
                 try:
                     val = md[key]
-                except:
+                except Exception:
                     val = None
                 kw[key] = val
             return self.ZCacheable_get(keywords=kw, default=default)
@@ -191,20 +215,20 @@ class DTMLMethod(RestrictedDTML,
             for key in self._cache_namespace_keys:
                 try:
                     val = md[key]
-                except:
+                except Exception:
                     val = None
                 kw[key] = val
             self.ZCacheable_set(result, keywords=kw)
 
-    security.declareProtected(change_dtml_methods, 'ZCacheable_configHTML')
+    security.declareProtected(change_dtml_methods, 'ZCacheable_configHTML')  # NOQA: D001,E501
     ZCacheable_configHTML = DTMLFile('dtml/cacheNamespaceKeys', globals())
 
-    security.declareProtected(change_dtml_methods, 'getCacheNamespaceKeys')
+    @security.protected(change_dtml_methods)
     def getCacheNamespaceKeys(self):
         # Return the cacheNamespaceKeys.
         return self._cache_namespace_keys
 
-    security.declareProtected(change_dtml_methods, 'setCacheNamespaceKeys')
+    @security.protected(change_dtml_methods)
     def setCacheNamespaceKeys(self, keys, REQUEST=None):
         # Set the list of names looked up to provide a cache key.
         ks = []
@@ -213,109 +237,87 @@ class DTMLMethod(RestrictedDTML,
             if key:
                 ks.append(key)
         self._cache_namespace_keys = tuple(ks)
+
         if REQUEST is not None:
             return self.ZCacheable_manage(self, REQUEST)
 
-    security.declareProtected(View, 'get_size')
+    @security.protected(View)
     def get_size(self):
         return len(self.raw)
 
     # deprecated; use get_size!
     getSize = get_size
 
-    security.declareProtected(change_dtml_methods, 'manage')
+    security.declareProtected(change_dtml_methods, 'manage')  # NOQA: D001
 
-    security.declareProtected(change_dtml_methods, 'manage_editForm')
+    security.declareProtected(change_dtml_methods, 'manage_editForm')  # NOQA: D001,E501
     manage_editForm = DTMLFile('dtml/documentEdit', globals())
     manage_editForm._setName('manage_editForm')
 
     # deprecated!
     manage_uploadForm = manage_editForm
 
-    security.declareProtected(change_dtml_methods, 'manage_main')
+    security.declareProtected(change_dtml_methods, 'manage_main')  # NOQA: D001
     manage = manage_main = manage_editDocument = manage_editForm
 
-    security.declareProtected(change_proxy_roles, 'manage_proxyForm')
+    security.declareProtected(change_proxy_roles, 'manage_proxyForm')  # NOQA: D001,E501
     manage_proxyForm = DTMLFile('dtml/documentProxy', globals())
 
-    _size_changes = {
-        'Bigger': (5, 5),
-        'Smaller': (-5, -5),
-        'Narrower': (0, -5),
-        'Wider': (0, 5),
-        'Taller': (5, 0),
-        'Shorter': (-5, 0),
-        }
-
-    def _er(self, data, title, SUBMIT, dtpref_cols, dtpref_rows, REQUEST):
-        dr, dc = self._size_changes[SUBMIT]
-        rows = str(max(1, int(dtpref_rows) + dr))
-        cols = str(dtpref_cols)
-        if cols.endswith('%'):
-            cols = str(min(100, max(25, int(cols[:-1]) + dc))) + '%'
-        else:
-            cols = str(max(35, int(cols) + dc))
-        e = (DateTime("GMT") + 365).rfc822()
-        setCookie = REQUEST["RESPONSE"].setCookie
-        setCookie("dtpref_rows", rows, path='/', expires=e)
-        setCookie("dtpref_cols", cols, path='/', expires=e)
-        REQUEST.other.update({"dtpref_cols": cols, "dtpref_rows": rows})
-        return self.manage_main(self, REQUEST, title=title,
-                                __str__=self.quotedHTML(data))
-
-    security.declareProtected(change_dtml_methods, 'manage_edit')
-    def manage_edit(self, data, title,
-                    SUBMIT='Change',
-                    dtpref_cols='100%',
-                    dtpref_rows='20',
-                    REQUEST=None):
+    @security.protected(change_dtml_methods)
+    def manage_edit(self, data, title, SUBMIT='Change', REQUEST=None):
         """ Replace contents with 'data', title with 'title'.
-
-        The SUBMIT parameter is also used to change the size of the editing
-        area on the default Document edit screen.  If the value is "Smaller",
-        the rows and columns decrease by 5.  If the value is "Bigger", the
-        rows and columns increase by 5.  If any other or no value is supplied,
-        the data gets checked for DTML errors and is saved.
         """
         self._validateProxy(REQUEST)
-        if self._size_changes.has_key(SUBMIT):
-            return self._er(data, title,
-                            SUBMIT, dtpref_cols, dtpref_rows, REQUEST)
         if self.wl_isLocked():
-            raise ResourceLockedError('This item is locked via WebDAV')
+            raise ResourceLockedError(self._locked_error_text)
 
         self.title = str(title)
         if isinstance(data, TaintedString):
             data = data.quoted()
-        if not isinstance(data, basestring):
+
+        if hasattr(data, 'read'):
             data = data.read()
-        self.munge(data)
+        try:
+            self.munge(data)
+        except ParseError as e:
+            if REQUEST:
+                return self.manage_main(
+                    self, REQUEST, manage_tabs_message=e,
+                    manage_tabs_type='warning')
+            else:
+                raise
         self.ZCacheable_invalidate()
         if REQUEST:
             message = "Saved changes."
             return self.manage_main(self, REQUEST, manage_tabs_message=message)
 
-    security.declareProtected(change_dtml_methods, 'manage_upload')
+    @security.protected(change_dtml_methods)
     def manage_upload(self, file='', REQUEST=None):
         """ Replace the contents of the document with the text in 'file'.
+
+        Store `file` as a native `str`.
         """
         self._validateProxy(REQUEST)
         if self.wl_isLocked():
-            raise ResourceLockedError('This DTML Method is locked via WebDAV')
+            if REQUEST is not None:
+                return self.manage_main(
+                    self, REQUEST,
+                    manage_tabs_message=self._locked_error_text,
+                    manage_tabs_type='warning')
+            raise ResourceLockedError(self._locked_error_text)
 
-        if type(file) is not str:
-            if REQUEST and not file:
-                raise ValueError('No file specified')
-            file = file.read()
+        if REQUEST is not None and not file:
+            return self.manage_main(
+                self, REQUEST,
+                manage_tabs_message='No file specified',
+                manage_tabs_type='warning')
 
-        self.munge(file)
+        self.munge(safe_file_data(file))
         self.ZCacheable_invalidate()
-        if REQUEST:
-            message = "Saved changes."
+        if REQUEST is not None:
+            message = "Content uploaded."
             return self.manage_main(self, REQUEST, manage_tabs_message=message)
 
-
-
     def manage_haveProxy(self, r):
         return r in self._proxy_roles
 
@@ -336,83 +338,72 @@ class DTMLMethod(RestrictedDTML,
 
         raise Forbidden(
             'You are not authorized to change <em>%s</em> because you '
-            'do not have proxy roles.\n<!--%s, %s-->'
-                % (self.__name__, u, roles))
+            'do not have proxy roles.\n<!--%s, %s-->' % (
+                self.__name__, u, roles))
 
-    security.declareProtected(change_proxy_roles, 'manage_proxy')
+    @security.protected(change_proxy_roles)
     @requestmethod('POST')
     def manage_proxy(self, roles=(), REQUEST=None):
         "Change Proxy Roles"
         self._validateProxy(REQUEST, roles)
         self._validateProxy(REQUEST)
         self._proxy_roles = tuple(roles)
-        self.ZCacheable_invalidate()
         if REQUEST:
             message = "Saved changes."
             return self.manage_proxyForm(self, REQUEST,
                                          manage_tabs_message=message)
 
-    security.declareProtected(view_management_screens, 'PrincipiaSearchSource')
+    @security.protected(view_management_screens)
     def PrincipiaSearchSource(self):
         # Support for searching - the document's contents are searched.
         return self.read()
 
-    security.declareProtected(view_management_screens, 'document_src')
+    @security.protected(view_management_screens)
     def document_src(self, REQUEST=None, RESPONSE=None):
         # Return unprocessed document source.
         if RESPONSE is not None:
             RESPONSE.setHeader('Content-Type', 'text/plain')
         return self.read()
 
-    ## Protocol handlers
+    if bbb.HAS_ZSERVER:
 
-    security.declareProtected(change_dtml_methods, 'PUT')
-    def PUT(self, REQUEST, RESPONSE):
-        """ Handle FTP / HTTP PUT requests.
-        """
-        self.dav__init(REQUEST, RESPONSE)
-        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
-        body = REQUEST.get('BODY', '')
-        self._validateProxy(REQUEST)
-        self.munge(body)
-        self.ZCacheable_invalidate()
-        RESPONSE.setStatus(204)
-        return RESPONSE
+        @security.protected(change_dtml_methods)
+        def PUT(self, REQUEST, RESPONSE):
+            """ Handle FTP / HTTP PUT requests.
+            """
+            self.dav__init(REQUEST, RESPONSE)
+            self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
+            body = REQUEST.get('BODY', '')
+            self._validateProxy(REQUEST)
+            self.munge(body)
+            self.ZCacheable_invalidate()
+            RESPONSE.setStatus(204)
+            return RESPONSE
 
-    security.declareProtected(ftp_access, 'manage_FTPstat')
-    security.declareProtected(ftp_access, 'manage_FTPlist')
+        @security.protected(ftp_access)
+        def manage_FTPget(self):
+            """ Get source for FTP download.
+            """
+            return self.read()
 
-    security.declareProtected(ftp_access, 'manage_FTPget')
-    def manage_FTPget(self):
-        """ Get source for FTP download.
-        """
-        return self.read()
-
-
-    def manage_historyCompare(self, rev1, rev2, REQUEST,
-                              historyComparisonResults=''):
-        return DTMLMethod.inheritedAttribute('manage_historyCompare')(
-            self, rev1, rev2, REQUEST,
-            historyComparisonResults = html_diff(rev1.read(), rev2.read()))
 
 InitializeClass(DTMLMethod)
 
-import re
-token = "[a-zA-Z0-9!#$%&'*+\-.\\\\^_`|~]+"
+token = r"[a-zA-Z0-9!#$%&'*+\-.\\\\^_`|~]+"
 hdr_start = re.compile(r'(%s):(.*)' % token).match
 
 
 def decapitate(html, RESPONSE=None):
     headers = []
-    spos  = 0
+    spos = 0
     eolen = 1
     while 1:
         m = hdr_start(html, spos)
         if not m:
-            if html[spos:spos+2] == '\r\n':
+            if html[spos:spos + 2] == '\r\n':
                 eolen = 2
                 break
-            if html[spos:spos+1] == '\n':
+            if html[spos:spos + 1] == '\n':
                 eolen = 1
                 break
             return html
@@ -421,10 +412,10 @@ def decapitate(html, RESPONSE=None):
         spos = m.end() + 1
         while spos < len(html) and html[spos] in ' \t':
             eol = html.find('\r\n', spos)
-            if eol <> -1:
+            if eol != -1:
                 eolen = 2
             else:
-                eol = html.find( '\n', spos)
+                eol = html.find('\n', spos)
                 if eol < 0:
                     return html
                 eolen = 1
@@ -437,39 +428,54 @@ def decapitate(html, RESPONSE=None):
     return html[spos + eolen:]
 
 
-default_dm_html="""<html>
-  <head><title><dtml-var title_or_id></title>
+def safe_file_data(data):
+    # Helper to convert upload file content into a safe value for saving
+    if hasattr(data, 'read'):
+        data = data.read()
+    if PY3 and isinstance(data, binary_type):
+        data = data.decode('utf-8')
+    if PY2 and isinstance(data, text_type):
+        data = data.encode('utf-8')
+    return data
+
+
+default_dm_html = """\
+<!DOCTYPE html>
+<html>
+  <head>
+    <title><dtml-var title_or_id></title>
+    <meta charset="utf-8" />
   </head>
-  <body bgcolor="#FFFFFF">
-<h2><dtml-var title_or_id> <dtml-var document_title></h2>
-<p>
-This is the <dtml-var document_id> Document
-in the <dtml-var title_and_id> Folder.
-</p>
+  <body>
+    <h2><dtml-var title_or_id> <dtml-var document_title></h2>
+    <p>
+    This is the <dtml-var document_id> Document
+    in the <dtml-var title_and_id> Folder.
+    </p>
   </body>
 </html>"""
 
-addForm=DTMLFile('dtml/methodAdd', globals())
+addForm = DTMLFile('dtml/methodAdd', globals())
+
 
 def addDTMLMethod(self, id, title='', file='', REQUEST=None, submit=None):
     """Add a DTML Method object with the contents of file. If
     'file' is empty, default document text is used.
     """
-    if type(file) is not str:
-        file = file.read()
-    if not file:
-        file = default_dm_html
+    data = safe_file_data(file)
+    if not data:
+        data = default_dm_html
     id = str(id)
     title = str(title)
-    ob = DTMLMethod(file, __name__=id)
+    ob = DTMLMethod(data, __name__=id)
     ob.title = title
     id = self._setObject(id, ob)
     if REQUEST is not None:
         try:
             u = self.DestinationURL()
-        except:
+        except Exception:
             u = REQUEST['URL1']
-        if submit == " Add and Edit ":
+        if submit == "Add and Edit":
             u = "%s/%s" % (u, quote(id))
-        REQUEST.RESPONSE.redirect(u+'/manage_main')
+        REQUEST.RESPONSE.redirect(u + '/manage_main')
     return ''
