@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Foundation and Contributors.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -11,32 +11,33 @@
 #
 ##############################################################################
 """Property management
-
-$Id: PropertyManager.py 94458 2009-01-02 19:29:55Z tseaver $
 """
 
-from cgi import escape
+import six
 
+from AccessControl.class_init import InitializeClass
 from AccessControl.Permissions import access_contents_information
 from AccessControl.Permissions import manage_properties
 from AccessControl.SecurityInfo import ClassSecurityInfo
 from Acquisition import aq_base
-from App.class_init import InitializeClass
-from ExtensionClass import Base
 from App.special_dtml import DTMLFile
-from App.Dialogs import MessageDialog
-from Persistence import Persistent
-from zExceptions import BadRequest
-from zope.interface import implements
-from ZPublisher.Converters import type_converters
-
+from ExtensionClass import Base
 from OFS.interfaces import IPropertyManager
 from OFS.PropertySheets import DefaultPropertySheets
 from OFS.PropertySheets import vps
-from OFS.ZDOM import ElementWithAttributes
+from zExceptions import BadRequest
+from zope.interface import implementer
+from ZPublisher.Converters import type_converters
 
-class PropertyManager(Base, ElementWithAttributes):
 
+try:
+    from html import escape
+except ImportError:  # PY2
+    from cgi import escape
+
+
+@implementer(IPropertyManager)
+class PropertyManager(Base):
     """
     The PropertyManager mixin class provides an object with
     transparent property management. An object which wants to
@@ -99,48 +100,57 @@ class PropertyManager(Base, ElementWithAttributes):
                              'manage_changeProperties',)),
     """
 
-    implements(IPropertyManager)
-
     security = ClassSecurityInfo()
     security.declareObjectProtected(access_contents_information)
     security.setPermissionDefault(access_contents_information,
                                   ('Anonymous', 'Manager'))
 
-    manage_options=(
-        {'label':'Properties', 'action':'manage_propertiesForm',
-         'help':('OFSP','Properties.stx')},
-        )
+    manage_options = (
+        {
+            'label': 'Properties',
+            'action': 'manage_propertiesForm',
+        },
+    )
 
-    security.declareProtected(manage_properties, 'manage_propertiesForm')
-    manage_propertiesForm=DTMLFile('dtml/properties', globals(),
-                                   property_extensible_schema__=1)
-    security.declareProtected(manage_properties, 'manage_propertyTypeForm')
-    manage_propertyTypeForm=DTMLFile('dtml/propertyType', globals())
+    security.declareProtected(manage_properties, 'manage_propertiesForm')  # NOQA: D001,E501
+    manage_propertiesForm = DTMLFile(
+        'dtml/properties', globals(), property_extensible_schema__=1)
+    security.declareProtected(manage_properties, 'manage_propertyTypeForm')  # NOQA: D001,E501
+    manage_propertyTypeForm = DTMLFile('dtml/propertyType', globals())
 
-    title=''
-    _properties=({'id':'title', 'type': 'string', 'mode':'wd'},)
-    _reserved_names=()
+    title = ''
+    _properties = (
+        {
+            'id': 'title',
+            'type': 'string',
+            'mode': 'wd',
+        },
+    )
+    _reserved_names = ()
 
-    __propsets__=()
-    propertysheets=vps(DefaultPropertySheets)
+    __propsets__ = ()
+    propertysheets = vps(DefaultPropertySheets)
 
-    security.declareProtected(access_contents_information, 'valid_property_id')
+    @security.protected(access_contents_information)
     def valid_property_id(self, id):
-        if not id or id[:1]=='_' or (id[:3]=='aq_') \
-           or (' ' in id) or hasattr(aq_base(self), id) or escape(id) != id:
+        if not id or \
+           id[:1] == '_' or \
+           id[:3] == 'aq_' or \
+           ' ' in id or \
+           hasattr(aq_base(self), id) or \
+           escape(id, True) != id:
             return 0
         return 1
 
-    security.declareProtected(access_contents_information, 'hasProperty')
+    @security.protected(access_contents_information)
     def hasProperty(self, id):
-        """Return true if object has a property 'id'.
-        """
+        """Return true if object has a property 'id'."""
         for p in self._properties:
-            if id==p['id']:
+            if id == p['id']:
                 return 1
         return 0
 
-    security.declareProtected(access_contents_information, 'getProperty')
+    @security.protected(access_contents_information)
     def getProperty(self, id, d=None):
         """Get the property 'id'.
 
@@ -151,31 +161,31 @@ class PropertyManager(Base, ElementWithAttributes):
             return getattr(self, id)
         return d
 
-    security.declareProtected(access_contents_information, 'getPropertyType')
+    @security.protected(access_contents_information)
     def getPropertyType(self, id):
         """Get the type of property 'id'.
 
         Returns None if no such property exists.
         """
         for md in self._properties:
-            if md['id']==id:
+            if md['id'] == id:
                 return md.get('type', 'string')
         return None
 
     def _wrapperCheck(self, object):
         # Raise an error if an object is wrapped.
         if hasattr(object, 'aq_base'):
-            raise ValueError, 'Invalid property value: wrapped object'
+            raise ValueError('Invalid property value: wrapped object')
         return
 
     def _setPropValue(self, id, value):
         self._wrapperCheck(value)
         if type(value) == list:
             value = tuple(value)
-        setattr(self,id,value)
+        setattr(self, id, value)
 
     def _delPropValue(self, id):
-        delattr(self,id)
+        delattr(self, id)
 
     def _setProperty(self, id, value, type='string'):
         # for selection and multiple selection properties
@@ -183,19 +193,19 @@ class PropertyManager(Base, ElementWithAttributes):
         # of the property
         self._wrapperCheck(value)
         if not self.valid_property_id(id):
-            raise BadRequest, 'Invalid or duplicate property id'
+            raise BadRequest('Invalid or duplicate property id')
 
         if type in ('selection', 'multiple selection'):
             if not hasattr(self, value):
-                raise BadRequest, 'No select variable %s' % value
-            self._properties=self._properties + (
-                {'id':id, 'type':type, 'select_variable':value},)
-            if type=='selection':
+                raise BadRequest('No select variable %s' % value)
+            self._properties = self._properties + (
+                {'id': id, 'type': type, 'select_variable': value},)
+            if type == 'selection':
                 self._setPropValue(id, '')
             else:
                 self._setPropValue(id, [])
         else:
-            self._properties=self._properties+({'id':id,'type':type},)
+            self._properties = self._properties + ({'id': id, 'type': type},)
             self._setPropValue(id, value)
 
     def _updateProperty(self, id, value):
@@ -204,52 +214,49 @@ class PropertyManager(Base, ElementWithAttributes):
         # the value to the type of the existing property.
         self._wrapperCheck(value)
         if not self.hasProperty(id):
-            raise BadRequest, 'The property %s does not exist' % escape(id)
-        if type(value)==type(''):
-            proptype=self.getPropertyType(id) or 'string'
-            if type_converters.has_key(proptype):
-                value=type_converters[proptype](value)
+            raise BadRequest(
+                'The property %s does not exist' % escape(id, True))
+        if isinstance(value, (six.string_types, six.binary_type)):
+            proptype = self.getPropertyType(id) or 'string'
+            if proptype in type_converters:
+                value = type_converters[proptype](value)
         self._setPropValue(id, value)
 
     def _delProperty(self, id):
         if not self.hasProperty(id):
-            raise ValueError, 'The property %s does not exist' % escape(id)
+            raise ValueError(
+                'The property %s does not exist' % escape(id, True))
         self._delPropValue(id)
-        self._properties=tuple(filter(lambda i, n=id: i['id'] != n,
-                                      self._properties))
+        self._properties = tuple(i for i in self._properties if i['id'] != id)
 
-    security.declareProtected(access_contents_information, 'propertyIds')
+    @security.protected(access_contents_information)
     def propertyIds(self):
-        """Return a list of property ids.
-        """
-        return map(lambda i: i['id'], self._properties)
+        """Return a list of property ids."""
+        return [i['id'] for i in self._properties]
 
-    security.declareProtected(access_contents_information, 'propertyValues')
+    @security.protected(access_contents_information)
     def propertyValues(self):
-        """Return a list of actual property objects.
-        """
-        return map(lambda i,s=self: getattr(s,i['id']), self._properties)
+        """Return a list of actual property objects."""
+        return [getattr(self, i['id']) for i in self._properties]
 
-    security.declareProtected(access_contents_information, 'propertyItems')
+    @security.protected(access_contents_information)
     def propertyItems(self):
-        """Return a list of (id,property) tuples.
-        """
-        return map(lambda i,s=self: (i['id'],getattr(s,i['id'])),
-                                    self._properties)
+        """Return a list of (id,property) tuples."""
+        return [(i['id'], getattr(self, i['id'])) for i in self._properties]
+
     def _propertyMap(self):
-        """Return a tuple of mappings, giving meta-data for properties.
-        """
+        """Return a tuple of mappings, giving meta-data for properties."""
         return self._properties
 
-    security.declareProtected(access_contents_information, 'propertyMap')
+    @security.protected(access_contents_information)
     def propertyMap(self):
         """Return a tuple of mappings, giving meta-data for properties.
 
         Return copies of the real definitions for security.
         """
-        return tuple(map(lambda dict: dict.copy(), self._propertyMap()))
+        return tuple(dict.copy() for dict in self._propertyMap())
 
-    security.declareProtected(access_contents_information, 'propertyLabel')
+    @security.protected(access_contents_information)
     def propertyLabel(self, id):
         """Return a label for the given property id
         """
@@ -258,8 +265,7 @@ class PropertyManager(Base, ElementWithAttributes):
                 return p.get('label', id)
         return id
 
-    security.declareProtected(access_contents_information,
-                              'propertyDescription')
+    @security.protected(access_contents_information)
     def propertyDescription(self, id):
         """Return a description for the given property id
         """
@@ -268,29 +274,26 @@ class PropertyManager(Base, ElementWithAttributes):
                 return p.get('description', '')
         return id
 
-    security.declareProtected(access_contents_information, 'propdict')
+    @security.protected(access_contents_information)
     def propdict(self):
-        dict={}
+        dict = {}
         for p in self._properties:
-            dict[p['id']]=p
+            dict[p['id']] = p
         return dict
 
-
-    # Web interface
-
-    security.declareProtected(manage_properties, 'manage_addProperty')
+    @security.protected(manage_properties)
     def manage_addProperty(self, id, value, type, REQUEST=None):
         """Add a new property via the web.
 
         Sets a new property with the given id, type, and value.
         """
-        if type_converters.has_key(type):
-            value=type_converters[type](value)
+        if type in type_converters:
+            value = type_converters[type](value)
         self._setProperty(id.strip(), value, type)
         if REQUEST is not None:
             return self.manage_propertiesForm(self, REQUEST)
 
-    security.declareProtected(manage_properties, 'manage_editProperties')
+    @security.protected(manage_properties)
     def manage_editProperties(self, REQUEST):
         """Edit object properties via the web.
 
@@ -300,47 +303,56 @@ class PropertyManager(Base, ElementWithAttributes):
         instead for most situations.
         """
         for prop in self._propertyMap():
-            name=prop['id']
+            name = prop['id']
             if 'w' in prop.get('mode', 'wd'):
                 if prop['type'] == 'multiple selection':
-                    value=REQUEST.get(name, [])
+                    value = REQUEST.form.get(name, [])
                 else:
-                    value=REQUEST.get(name, '')
+                    value = REQUEST.form.get(name, '')
                 self._updateProperty(name, value)
         if REQUEST:
-            message="Saved changes."
-            return self.manage_propertiesForm(self,REQUEST,
-                                              manage_tabs_message=message)
+            message = "Saved changes."
+            return self.manage_propertiesForm(
+                self,
+                REQUEST,
+                manage_tabs_message=message,
+            )
 
-    security.declareProtected(manage_properties, 'manage_changeProperties')
+    @security.protected(manage_properties)
     def manage_changeProperties(self, REQUEST=None, **kw):
         """Change existing object properties.
 
-        Change object properties by passing either a mapping object
-        of name:value pairs {'foo':6} or passing name=value parameters
+        Change object properties by passing either a REQUEST object or
+        name=value parameters
         """
         if REQUEST is None:
-            props={}
-        else: props=REQUEST
+            props = {}
+        elif isinstance(REQUEST, dict):
+            props = REQUEST
+        else:
+            props = REQUEST.form
         if kw:
             for name, value in kw.items():
-                props[name]=value
-        propdict=self.propdict()
+                props[name] = value
+        propdict = self.propdict()
         for name, value in props.items():
             if self.hasProperty(name):
-                if not 'w' in propdict[name].get('mode', 'wd'):
-                    raise BadRequest, '%s cannot be changed' % escape(name)
+                if 'w' not in propdict[name].get('mode', 'wd'):
+                    raise BadRequest(
+                        '%s cannot be changed' % escape(name, True))
                 self._updateProperty(name, value)
+
         if REQUEST:
-            message="Saved changes."
-            return self.manage_propertiesForm(self,REQUEST,
-                                              manage_tabs_message=message)
+            message = "Saved changes."
+            return self.manage_propertiesForm(
+                self,
+                REQUEST,
+                manage_tabs_message=message
+            )
 
-    # Note - this is experimental, pending some community input.
-
-    security.declareProtected(manage_properties, 'manage_changePropertyTypes')
+    @security.protected(manage_properties)
     def manage_changePropertyTypes(self, old_ids, props, REQUEST=None):
-        """Replace one set of properties with another
+        """Replace one set of properties with another.
 
         Delete all properties that have ids in old_ids, then add a
         property for each item in props.  Each item has a new_id,
@@ -357,33 +369,29 @@ class PropertyManager(Base, ElementWithAttributes):
         if REQUEST is not None:
             return self.manage_propertiesForm(self, REQUEST)
 
-
-    security.declareProtected(manage_properties, 'manage_delProperties')
+    @security.protected(manage_properties)
     def manage_delProperties(self, ids=None, REQUEST=None):
         """Delete one or more properties specified by 'ids'."""
         if REQUEST:
             # Bugfix for property named "ids" (Casey)
-            if ids == self.getProperty('ids', None): ids = None
+            if ids == self.getProperty('ids', None):
+                ids = None
             ids = REQUEST.get('_ids', ids)
         if ids is None:
-            return MessageDialog(
-                   title='No property specified',
-                   message='No properties were specified!',
-                   action ='./manage_propertiesForm',)
-        propdict=self.propdict()
-        nd=self._reserved_names
+            raise BadRequest('No property specified.')
+        propdict = self.propdict()
+        nd = self._reserved_names
         for id in ids:
             if not hasattr(aq_base(self), id):
-                raise BadRequest, (
-                      'The property <em>%s</em> does not exist' % escape(id))
-            if (not 'd' in propdict[id].get('mode', 'wd')) or (id in nd):
-                return MessageDialog(
-                title  ='Cannot delete %s' % id,
-                message='The property <em>%s</em> cannot be deleted.' % escape(id),
-                action ='manage_propertiesForm')
+                raise BadRequest(
+                    ('The property <em>%s</em> '
+                     'does not exist' % escape(id, True)))
+            if ('d' not in propdict[id].get('mode', 'wd')) or (id in nd):
+                raise BadRequest('Cannot delete %s' % id)
             self._delProperty(id)
 
         if REQUEST is not None:
             return self.manage_propertiesForm(self, REQUEST)
 
+
 InitializeClass(PropertyManager)
