@@ -13,7 +13,7 @@
 """This module implements a mix-in for traversable objects.
 """
 
-from urllib import quote
+from six.moves.urllib.parse import quote
 
 from AccessControl.class_init import InitializeClass
 from AccessControl.SecurityInfo import ClassSecurityInfo
@@ -26,27 +26,29 @@ from Acquisition import aq_base
 from Acquisition import aq_inner
 from Acquisition import aq_parent
 from Acquisition.interfaces import IAcquirer
+from OFS import bbb
+from OFS.interfaces import IApplication
 from OFS.interfaces import ITraversable
 from zExceptions import NotFound
-from ZPublisher.interfaces import UseTraversalDefault
 from ZODB.POSException import ConflictError
-
-from zope.interface import implements
-from zope.interface import Interface
 from zope.component import queryMultiAdapter
+from zope.interface import Interface
+from zope.interface import implementer
 from zope.location.interfaces import LocationError
 from zope.traversing.namespace import namespaceLookup
 from zope.traversing.namespace import nsParse
+from ZPublisher.interfaces import UseTraversalDefault
+
 
 _marker = object()
 
 
-class Traversable:
-    implements(ITraversable)
+@implementer(ITraversable)
+class Traversable(object):
 
     security = ClassSecurityInfo()
 
-    security.declarePublic('absolute_url')
+    @security.public
     def absolute_url(self, relative=0):
         """Return the absolute URL of the object.
 
@@ -74,7 +76,7 @@ class Traversable:
             return path2url(spp[1:])
         return toUrl(spp)
 
-    security.declarePublic('absolute_url_path')
+    @security.public
     def absolute_url_path(self):
         """Return the path portion of the absolute URL of the object.
 
@@ -88,7 +90,7 @@ class Traversable:
             return path2url(spp) or '/'
         return toUrl(spp, relative=1) or '/'
 
-    security.declarePublic('virtual_url_path')
+    @security.public
     def virtual_url_path(self):
         """Return a URL for the object, relative to the site root.
 
@@ -103,10 +105,11 @@ class Traversable:
             return path2url(spp[1:])
         return path2url(toVirt(spp))
 
-    security.declarePrivate('getPhysicalRoot')
-    getPhysicalRoot=Acquired
+    # decorators did not work on variables
+    security.declarePrivate('getPhysicalRoot')  # NOQA: D001
+    getPhysicalRoot = Acquired
 
-    security.declarePublic('getPhysicalPath')
+    @security.public
     def getPhysicalPath(self):
         # Get the physical path of the object.
         #
@@ -115,15 +118,38 @@ class Traversable:
         # operation.  getPhysicalRoot() and getPhysicalPath() are designed to
         # operate together.
 
-        path = (self.getId(),)
+        # This implementation is optimized to avoid excessive amounts of
+        # function calls while walking up from an object on a deep level.
+        try:
+            id = self.id or self.getId()
+        except AttributeError:
+            id = self.getId()
 
+        path = (id, )
         p = aq_parent(aq_inner(self))
-        if p is not None:
-            path = p.getPhysicalPath() + path
+        if p is None:
+            return path
+
+        func = self.getPhysicalPath.__func__
+        while p is not None:
+            if func is p.getPhysicalPath.__func__:
+                try:
+                    pid = p.id or p.getId()
+                except AttributeError:
+                    pid = p.getId()
+
+                path = (pid, ) + path
+                p = aq_parent(aq_inner(p))
+            else:
+                if IApplication.providedBy(p):
+                    path = ('', ) + path
+                else:
+                    path = p.getPhysicalPath() + path
+                break
 
         return path
 
-    security.declarePrivate('unrestrictedTraverse')
+    @security.private
     def unrestrictedTraverse(self, path, default=_marker, restricted=False):
         """Lookup an object by path.
 
@@ -138,9 +164,10 @@ class Traversable:
         be traversed for any reason (i.e., no object exists at that path or
         the object is inaccessible).
 
-        restricted -- If false (default) then no security checking is performed.
-        If true, then all of the objects along the path are validated with
-        the security machinery. Usually invoked using restrictedTraverse().
+        restricted -- If false (default) then no security checking is
+        performed. If true, then all of the objects along the path are
+        validated with the security machinery. Usually invoked using
+        restrictedTraverse().
         """
         if not path:
             return self
@@ -167,12 +194,16 @@ class Traversable:
             path_pop()
             obj = self.getPhysicalRoot()
             if restricted:
-                validate(None, None, None, obj) # may raise Unauthorized
+                validate(None, None, None, obj)  # may raise Unauthorized
         else:
             obj = self
 
         # import time ordering problem
-        from webdav.NullResource import NullResource
+        if bbb.HAS_ZSERVER:
+            from webdav.NullResource import NullResource
+        else:
+            NullResource = bbb.NullResource
+
         resource = _marker
         try:
             while path:
@@ -181,7 +212,7 @@ class Traversable:
 
                 if name[0] == '_':
                     # Never allowed in a URL.
-                    raise NotFound, name
+                    raise NotFound(name)
 
                 if name == '..':
                     next = aq_parent(obj)
@@ -193,7 +224,12 @@ class Traversable:
 
                 bobo_traverse = getattr(obj, '__bobo_traverse__', None)
                 try:
-                    if name and name[:1] in '@+' and name != '+' and nsParse(name)[1]:
+                    if (
+                        name
+                        and name[:1] in '@+'
+                        and name != '+'
+                        and nsParse(name)[1]
+                    ):
                         # Process URI segment parameters.
                         ns, nm = nsParse(name)
                         try:
@@ -202,51 +238,60 @@ class Traversable:
                             if IAcquirer.providedBy(next):
                                 next = next.__of__(obj)
                             if restricted and not validate(
-                                obj, obj, name, next):
+                                    obj, obj, name, next):
                                 raise Unauthorized(name)
                         except LocationError:
                             raise AttributeError(name)
 
                     else:
-                        next = UseTraversalDefault # indicator
+                        next = UseTraversalDefault  # indicator
                         try:
                             if bobo_traverse is not None:
                                 next = bobo_traverse(REQUEST, name)
                                 if restricted:
                                     if aq_base(next) is not next:
-                                        # The object is wrapped, so the acquisition
-                                        # context is the container.
+                                        # The object is wrapped, so the
+                                        # acquisition context is the container.
                                         container = aq_parent(aq_inner(next))
-                                    elif getattr(next, 'im_self', None) is not None:
+                                    elif getattr(next, '__self__',
+                                                 None) is not None:
                                         # Bound method, the bound instance
                                         # is the container
-                                        container = next.im_self
-                                    elif getattr(aq_base(obj), name, _marker) is next:
-                                        # Unwrapped direct attribute of the object so
-                                        # object is the container
+                                        container = next.__self__
+                                    elif getattr(
+                                            aq_base(obj),
+                                            name, _marker) is next:
+                                        # Unwrapped direct attribute of the
+                                        # object so object is the container
                                         container = obj
                                     else:
                                         # Can't determine container
                                         container = None
-                                    # If next is a simple unwrapped property, its
-                                    # parentage is indeterminate, but it may have
-                                    # been acquired safely. In this case validate
-                                    # will raise an error, and we can explicitly
-                                    # check that our value was acquired safely.
+                                    # If next is a simple unwrapped property,
+                                    # its parentage is indeterminate, but it
+                                    # may have been acquired safely. In this
+                                    # case validate will raise an error, and
+                                    # we can explicitly check that our value
+                                    # was acquired safely.
                                     try:
-                                        ok = validate(obj, container, name, next)
+                                        ok = validate(
+                                            obj, container, name, next)
                                     except Unauthorized:
                                         ok = False
                                     if not ok:
-                                        if (container is not None or
-                                            guarded_getattr(obj, name, _marker)
-                                                is not next):
+                                        if (
+                                            container is not None
+                                            or guarded_getattr(obj, name, _marker) is not next  # NOQA: E501
+                                        ):
                                             raise Unauthorized(name)
                         except UseTraversalDefault:
-                            # behave as if there had been no '__bobo_traverse__'
+                            # behave as if there had been no
+                            # '__bobo_traverse__'
                             bobo_traverse = None
                         if next is UseTraversalDefault:
-                            if getattr(aq_base(obj), name, _marker) is not _marker:
+                            if getattr(
+                                    aq_base(obj),
+                                    name, _marker) is not _marker:
                                 if restricted:
                                     next = guarded_getattr(obj, name)
                                 else:
@@ -254,10 +299,14 @@ class Traversable:
                             else:
                                 try:
                                     next = obj[name]
-                                    # The item lookup may return a NullResource,
-                                    # if this is the case we save it and return it
-                                    # if all other lookups fail.
-                                    if isinstance(next, NullResource):
+                                    # The item lookup may return a
+                                    # NullResource, if this is the case we
+                                    # save it and return it if all other
+                                    # lookups fail.
+                                    if (
+                                        NullResource is not None
+                                        and isinstance(next, NullResource)
+                                    ):
                                         resource = next
                                         raise KeyError(name)
                                 except (AttributeError, TypeError):
@@ -266,13 +315,14 @@ class Traversable:
                                     # or TypeError: not subscriptable
                                     raise NotFound(name)
                                 if restricted and not validate(
-                                    obj, obj, None, next):
+                                        obj, obj, None, next):
                                     raise Unauthorized(name)
 
-                except (AttributeError, NotFound, KeyError), e:
+                except (AttributeError, NotFound, KeyError) as e:
                     # Try to look for a view
-                    next = queryMultiAdapter((obj, aq_acquire(self, 'REQUEST')),
-                                             Interface, name)
+                    next = queryMultiAdapter(
+                        (obj, aq_acquire(self, 'REQUEST')),
+                        Interface, name)
 
                     if next is not None:
                         if IAcquirer.providedBy(next):
@@ -305,17 +355,18 @@ class Traversable:
 
         except ConflictError:
             raise
-        except:
+        except Exception:
             if default is not _marker:
                 return default
             else:
                 raise
 
-    security.declarePublic('restrictedTraverse')
+    @security.public
     def restrictedTraverse(self, path, default=_marker):
         # Trusted code traversal code, always enforces securitys
         return self.unrestrictedTraverse(path, default, restricted=True)
 
+
 InitializeClass(Traversable)
 
 
