@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Corporation and Contributors.
+# Copyright (c) 2005,2010 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,31 +12,29 @@
 #
 ##############################################################################
 """
-Five subscriber definitions.
-
-$Id: subscribers.py 100339 2009-05-24 17:24:00Z hannosch $
+Subscriber definitions.
 """
 
-import warnings
 from logging import getLogger
 
 import OFS.interfaces
-from Acquisition import aq_base
-from App.config import getConfiguration
-from AccessControl import getSecurityManager
-from ZODB.POSException import ConflictError
-
 import zope.component
 import zope.interface
 import zope.location.interfaces
+from AccessControl import getSecurityManager
+from Acquisition import aq_base
+from App.config import getConfiguration
+from ZODB.POSException import ConflictError
 from zope.container.contained import dispatchToSublocations
-from zope.lifecycleevent.interfaces import IObjectMovedEvent
 from zope.lifecycleevent.interfaces import IObjectCopiedEvent
+from zope.lifecycleevent.interfaces import IObjectMovedEvent
+
 
 deprecatedManageAddDeleteClasses = []
 
 LOG = getLogger('OFS.subscribers')
 
+
 def compatibilityCall(method_name, *args):
     """Call a method if events have not been setup yet.
 
@@ -53,32 +51,29 @@ def compatibilityCall(method_name, *args):
     else:
         callManageAfterClone(*args)
 
+
 def maybeWarnDeprecated(ob, method_name):
     """Send a warning if a method is deprecated.
     """
     if not deprecatedManageAddDeleteClasses:
         # Directives not fully loaded
         return
-    for cls in deprecatedManageAddDeleteClasses:
-        if isinstance(ob, cls):
-            # Already deprecated through zcml
-            return
     if getattr(getattr(ob, method_name), '__five_method__', False):
         # Method knows it's deprecated
         return
+    if isinstance(ob, tuple(deprecatedManageAddDeleteClasses)):
+        return
     class_ = ob.__class__
-    warnings.warn(
+    LOG.debug(
         "%s.%s.%s is discouraged. You should use event subscribers instead." %
-        (class_.__module__, class_.__name__, method_name),
-        DeprecationWarning)
+        (class_.__module__, class_.__name__, method_name))
 
-##################################################
 
+@zope.interface.implementer(zope.location.interfaces.ISublocations)
+@zope.component.adapter(OFS.interfaces.IObjectManager)
 class ObjectManagerSublocations(object):
     """Get the sublocations for an ObjectManager.
     """
-    zope.component.adapts(OFS.interfaces.IObjectManager)
-    zope.interface.implements(zope.location.interfaces.ISublocations)
 
     def __init__(self, container):
         self.container = container
@@ -97,6 +92,7 @@ class ObjectManagerSublocations(object):
 # could have a simple subscriber for IObjectManager that directly calls
 # dispatchToSublocations.
 
+
 @zope.component.adapter(OFS.interfaces.IItem,
                         OFS.interfaces.IObjectWillBeMovedEvent)
 def dispatchObjectWillBeMovedEvent(ob, event):
@@ -108,6 +104,7 @@ def dispatchObjectWillBeMovedEvent(ob, event):
     # Next, do the manage_beforeDelete dance
     callManageBeforeDelete(ob, event.object, event.oldParent)
 
+
 @zope.component.adapter(OFS.interfaces.IItem, IObjectMovedEvent)
 def dispatchObjectMovedEvent(ob, event):
     """Multi-subscriber for IItem + IObjectMovedEvent.
@@ -118,6 +115,7 @@ def dispatchObjectMovedEvent(ob, event):
     if OFS.interfaces.IObjectManager.providedBy(ob):
         dispatchToSublocations(ob, event)
 
+
 @zope.component.adapter(OFS.interfaces.IItem,
                         OFS.interfaces.IObjectClonedEvent)
 def dispatchObjectClonedEvent(ob, event):
@@ -129,6 +127,7 @@ def dispatchObjectClonedEvent(ob, event):
     if OFS.interfaces.IObjectManager.providedBy(ob):
         dispatchToSublocations(ob, event)
 
+
 @zope.component.adapter(OFS.interfaces.IItem, IObjectCopiedEvent)
 def dispatchObjectCopiedEvent(ob, event):
     """Multi-subscriber for IItem + IObjectCopiedEvent.
@@ -148,6 +147,7 @@ def callManageAfterAdd(ob, item, container):
     maybeWarnDeprecated(ob, 'manage_afterAdd')
     ob.manage_afterAdd(item, container)
 
+
 def callManageBeforeDelete(ob, item, container):
     """Compatibility subscriber for manage_beforeDelete.
     """
@@ -156,20 +156,21 @@ def callManageBeforeDelete(ob, item, container):
     if getattr(aq_base(ob), 'manage_beforeDelete', None) is None:
         return
     maybeWarnDeprecated(ob, 'manage_beforeDelete')
-    import OFS.ObjectManager # avoid circular imports
+    import OFS.ObjectManager  # avoid circular imports
     try:
         ob.manage_beforeDelete(item, container)
     except OFS.ObjectManager.BeforeDeleteException:
         raise
     except ConflictError:
         raise
-    except:
+    except Exception:
         LOG.error('_delObject() threw', exc_info=True)
         # In debug mode when non-Manager, let exceptions propagate.
         if getConfiguration().debug_mode:
             if not getSecurityManager().getUser().has_role('Manager'):
                 raise
 
+
 def callManageAfterClone(ob, item):
     """Compatibility subscriber for manage_afterClone.
     """
