@@ -1,6 +1,7 @@
-from Testing.ZopeTestCase import FunctionalTestCase
 import unittest
 
+from Testing.ZopeTestCase import FunctionalTestCase
+
 
 class ApplicationTests(unittest.TestCase):
 
@@ -30,17 +31,17 @@ class ApplicationTests(unittest.TestCase):
 
     def test_id_no_request(self):
         app = self._makeOne()
-        self.assertEqual(app.id(), 'Zope')
+        self.assertEqual(app.getId(), 'Zope')
 
     def test_id_w_request_no_SCRIPT_NAME(self):
         app = self._makeOne()
         app.REQUEST = {}
-        self.assertEqual(app.id(), 'Zope')
+        self.assertEqual(app.getId(), 'Zope')
 
     def test_id_w_request_w_SCRIPT_NAME(self):
         app = self._makeOne()
         app.REQUEST = {'SCRIPT_NAME': '/Dummy'}
-        self.assertEqual(app.id(), 'Dummy')
+        self.assertEqual(app.getId(), 'Dummy')
 
     def test_title_and_id_plus_title_or_id(self):
         app = self._makeOne()
@@ -48,27 +49,27 @@ class ApplicationTests(unittest.TestCase):
         self.assertEqual(app.title_and_id(), 'Other')
         self.assertEqual(app.title_or_id(), 'Other')
 
-    def test___bobo_traverse__attribute_hit(self):
+    def test_bobo_traverse_attribute_hit(self):
         app = self._makeOne()
         app.NAME = 'attribute'
         app._getOb = lambda x, y: x
         request = {}
         self.assertEqual(app.__bobo_traverse__(request, 'NAME'), 'attribute')
 
-    def test___bobo_traverse__attribute_miss_key_hit(self):
+    def test_bobo_traverse_attribute_miss_key_hit(self):
         app = self._makeOne()
         app._getOb = lambda x, y: x
         app._objects = [{'id': 'OTHER', 'meta_type': None}]
         request = {}
         self.assertEqual(app.__bobo_traverse__(request, 'OTHER'), 'OTHER')
 
-    def test___bobo_traverse__attribute_key_miss_R_M_default_real_request(self):
-        from UserDict import UserDict
+    def test_bobo_traverse_attribute_key_miss_R_M_default_real_request(self):
+        from six.moves import UserDict
         request = UserDict()
 
-        class _Response:
+        class _Response(object):
             def notFoundError(self, msg):
-                1/0
+                1 / 0
 
         request.RESPONSE = _Response()
         app = self._makeOne()
@@ -77,23 +78,31 @@ class ApplicationTests(unittest.TestCase):
         self.assertRaises(ZeroDivisionError,
                           app.__bobo_traverse__, request, 'NONESUCH')
 
-    def test___bobo_traverse__attribute_key_miss_R_M_default_fake_request(self):
+    def test_bobo_traverse_attribute_key_miss_R_M_default_fake_request(self):
         app = self._makeOne()
 
         app._getOb = _noWay
         request = {}
         self.assertRaises(KeyError, app.__bobo_traverse__, request, 'NONESUCH')
 
-    def test___bobo_traverse__attribute_key_miss_R_M_is_GET(self):
+    def test_bobo_traverse_attribute_key_miss_R_M_is_GET(self):
         app = self._makeOne()
 
         app._getOb = _noWay
         request = {'REQUEST_METHOD': 'GET'}
         self.assertRaises(KeyError, app.__bobo_traverse__, request, 'NONESUCH')
 
-    def test___bobo_traverse__attribute_key_miss_R_M_not_GET_POST(self):
+    def test_bobo_traverse_attribute_key_miss_R_M_not_GET_POST(self):
+        from OFS import bbb
+        if bbb.HAS_ZSERVER:
+            from webdav.NullResource import NullResource
+        else:
+            NullResource = bbb.NullResource
+
+        if NullResource is None:
+            return
+
         from Acquisition import aq_inner, aq_parent
-        from webdav.NullResource import NullResource
 
         app = self._makeOne()
         app._getOb = _noWay
@@ -101,7 +110,7 @@ class ApplicationTests(unittest.TestCase):
 
         result = app.__bobo_traverse__(request, 'OTHER')
 
-        self.assertTrue(isinstance(result, NullResource))
+        self.assertIsInstance(result, NullResource)
         self.assertTrue(aq_parent(aq_inner(result)) is app)
 
     def test_redirect_regression(self):
@@ -113,7 +122,7 @@ class ApplicationTests(unittest.TestCase):
         """
         from zExceptions import Redirect as RedirectException
         app = self._makeOne()
-        for name in ('Redirect', 'ZopeRedirect', 'PrincipiaRedirect'):
+        for name in ('Redirect', 'ZopeRedirect'):
             method = getattr(app, name, None)
             if method is None:
                 continue
@@ -130,8 +139,7 @@ class ApplicationPublishTests(FunctionalTestCase):
         This is part of PloneHotfix20171128.
         """
         # These are all aliases.
-        # PrincipiaRedirect is no longer there in Zope 4.
-        for name in ('Redirect', 'ZopeRedirect', 'PrincipiaRedirect'):
+        for name in ('Redirect', 'ZopeRedirect'):
             response = self.publish(
                 '/{0}?destination=http://google.nl'.format(name))
             # This should *not* return a 302 Redirect.
@@ -140,9 +148,3 @@ class ApplicationPublishTests(FunctionalTestCase):
 
 def _noWay(self, key, default=None):
     raise KeyError(key)
-
-def test_suite():
-    return unittest.TestSuite((
-        unittest.makeSuite(ApplicationTests),
-        unittest.makeSuite(ApplicationPublishTests),
-        ))
