@@ -1,59 +1,60 @@
-import unittest
-
-import Testing
-import Zope2
-Zope2.startup()
-
-import os, sys
+# -*- coding: utf-8 -*-
+import os
+import sys
 import time
-from cStringIO import StringIO
+import unittest
+from io import BytesIO
 
-from Acquisition import aq_base
-
-from OFS.Application import Application
-from OFS.SimpleItem import SimpleItem
-from OFS.Cache import ZCM_MANAGERS
-from OFS.Image import Pdata
-from ZPublisher.HTTPRequest import HTTPRequest
-from ZPublisher.HTTPResponse import HTTPResponse
-from App.Common import rfc1123_date
-from Testing.makerequest import makerequest
-from zExceptions import Redirect
-import transaction
+import six
+from six import PY3
 
 import OFS.Image
-
+import Testing.testbrowser
+import Testing.ZopeTestCase
+import transaction
+import Zope2
+from Acquisition import aq_base
+from App.Common import rfc1123_date
+from OFS.Application import Application
+from OFS.Cache import ZCM_MANAGERS
+from OFS.Image import Pdata
+from OFS.SimpleItem import SimpleItem
+from Testing.makerequest import makerequest
+from zExceptions import Redirect
 from zope.component import adapter
-from zope.lifecycleevent.interfaces import IObjectModifiedEvent
 from zope.lifecycleevent.interfaces import IObjectCreatedEvent
+from zope.lifecycleevent.interfaces import IObjectModifiedEvent
+from ZPublisher.HTTPRequest import HTTPRequest
+from ZPublisher.HTTPResponse import HTTPResponse
 
-try:
-    here = os.path.dirname(os.path.abspath(__file__))
-except:
-    here = os.path.dirname(os.path.abspath(sys.argv[0]))
 
-imagedata = os.path.join(here, 'test.gif')
+here = os.path.dirname(os.path.abspath(__file__))
 filedata = os.path.join(here, 'test.gif')
 
+Zope2.startup_wsgi()
+
+
 def makeConnection():
     import ZODB
     from ZODB.DemoStorage import DemoStorage
 
     s = DemoStorage()
-    return ZODB.DB( s ).open()
+    return ZODB.DB(s).open()
 
 
 def aputrequest(file, content_type):
     resp = HTTPResponse(stdout=sys.stdout)
     environ = {}
-    environ['SERVER_NAME']='foo'
-    environ['SERVER_PORT']='80'
+    environ['SERVER_NAME'] = 'foo'
+    environ['SERVER_PORT'] = '80'
     environ['REQUEST_METHOD'] = 'PUT'
     environ['CONTENT_TYPE'] = content_type
     req = HTTPRequest(stdin=file, environ=environ, response=resp)
     return req
 
-class DummyCache:
+
+class DummyCache(object):
+
     def __init__(self):
         self.clear()
 
@@ -71,8 +72,8 @@ class DummyCache:
         self.invalidated = ob
 
     def clear(self):
-        self.set=None
-        self.get=None
+        self.set = None
+        self.get = None
         self.invalidated = None
         self.si = None
 
@@ -80,60 +81,62 @@ class DummyCache:
         self.si = si
 
 
-ADummyCache=DummyCache()
+ADummyCache = DummyCache()
+
 
 class DummyCacheManager(SimpleItem):
     def ZCacheManager_getCache(self):
         return ADummyCache
 
+
 class EventCatcher(object):
-    
+
     def __init__(self):
         self.created = []
         self.modified = []
-        
         self.setUp()
-    
+
     def setUp(self):
         from zope.component import provideHandler
         provideHandler(self.handleCreated)
         provideHandler(self.handleModified)
-    
+
     def tearDown(self):
         from zope.component import getSiteManager
         getSiteManager().unregisterHandler(self.handleCreated)
         getSiteManager().unregisterHandler(self.handleModified)
-    
+
     def reset(self):
         self.created = []
         self.modified = []
-    
+
     @adapter(IObjectCreatedEvent)
     def handleCreated(self, event):
         if isinstance(event.object, OFS.Image.File):
             self.created.append(event)
-    
+
     @adapter(IObjectModifiedEvent)
     def handleModified(self, event):
         if isinstance(event.object, OFS.Image.File):
             self.modified.append(event)
 
+
 class FileTests(unittest.TestCase):
-    data = open(filedata, 'rb').read()
     content_type = 'application/octet-stream'
     factory = 'manage_addFile'
-    def setUp( self ):
 
+    def setUp(self):
+        with open(filedata, 'rb') as fd:
+            self.data = fd.read()
         self.connection = makeConnection()
         self.eventCatcher = EventCatcher()
-        
         try:
             r = self.connection.root()
             a = Application()
             r['Application'] = a
             self.root = a
-            responseOut = self.responseOut = StringIO()
-            self.app = makerequest( self.root, stdout=responseOut )
+            responseOut = self.responseOut = BytesIO()
+            self.app = makerequest(self.root, stdout=responseOut)
             self.app.dcm = DummyCacheManager()
             factory = getattr(self.app, self.factory)
             factory('file',
@@ -144,22 +147,25 @@ class FileTests(unittest.TestCase):
             # Hack, we need a _p_mtime for the file, so we make sure that it
             # has one.
             transaction.commit()
-        except:
+        except Exception:
+            transaction.abort()
             self.connection.close()
             raise
         transaction.begin()
-        self.file = getattr( self.app, 'file' )
-        
+        self.file = getattr(self.app, 'file')
+
         # Since we do the create here, let's test the events here too
-        self.assertEquals(1, len(self.eventCatcher.created))
-        self.assertTrue(aq_base(self.eventCatcher.created[0].object) is aq_base(self.file))
-        
-        self.assertEquals(1, len(self.eventCatcher.modified))
-        self.assertTrue(aq_base(self.eventCatcher.created[0].object) is aq_base(self.file))
-        
+        self.assertEqual(1, len(self.eventCatcher.created))
+        self.assertTrue(
+            aq_base(self.eventCatcher.created[0].object) is aq_base(self.file))
+
+        self.assertEqual(1, len(self.eventCatcher.modified))
+        self.assertTrue(
+            aq_base(self.eventCatcher.created[0].object) is aq_base(self.file))
+
         self.eventCatcher.reset()
-        
-    def tearDown( self ):
+
+    def tearDown(self):
         del self.file
         transaction.abort()
         self.connection.close()
@@ -168,45 +174,43 @@ class FileTests(unittest.TestCase):
         del self.root
         del self.connection
         ADummyCache.clear()
-        
         self.eventCatcher.tearDown()
 
     def testViewImageOrFile(self):
         self.assertRaises(Redirect, self.file.view_image_or_file, 'foo')
 
     def testUpdateData(self):
-        self.file.update_data('foo')
+        self.file.update_data(b'foo')
         self.assertEqual(self.file.size, 3)
-        self.assertEqual(self.file.data, 'foo')
+        self.assertEqual(self.file.data, b'foo')
         self.assertTrue(ADummyCache.invalidated)
         self.assertTrue(ADummyCache.set)
 
     def testReadData(self):
-        s = "a" * (2 << 16)
-        f = StringIO(s)
-        data, size = self.file._read_data(f)
-        self.assertTrue(isinstance(data, Pdata))
-        self.assertEqual(str(data), s)
-        self.assertEqual(len(s), len(str(data)))
+        s = b'a' * (2 << 16)
+        data, size = self.file._read_data(BytesIO(s))
+        self.assertIsInstance(data, Pdata)
+        self.assertEqual(bytes(data), s)
+        self.assertEqual(len(s), len(bytes(data)))
         self.assertEqual(len(s), size)
 
     def testBigPdata(self):
         # Test that a big enough string is split into several Pdata
         # From a file
-        s = "a" * (1 << 16) * 3
-        data, size = self.file._read_data(StringIO(s))
+        s = b'a' * (1 << 16) * 3
+        data, size = self.file._read_data(BytesIO(s))
         self.assertNotEqual(data.next, None)
         # From a string
         data, size = self.file._read_data(s)
         self.assertNotEqual(data.next, None)
 
     def testManageEditWithFileData(self):
-        self.file.manage_edit('foobar', 'text/plain', filedata='ASD')
+        self.file.manage_edit('foobar', 'text/plain', filedata=b'ASD')
         self.assertEqual(self.file.title, 'foobar')
         self.assertEqual(self.file.content_type, 'text/plain')
         self.assertTrue(ADummyCache.invalidated)
         self.assertTrue(ADummyCache.set)
-        self.assertEquals(1, len(self.eventCatcher.modified))
+        self.assertEqual(1, len(self.eventCatcher.modified))
         self.assertTrue(self.eventCatcher.modified[0].object is self.file)
 
     def testManageEditWithoutFileData(self):
@@ -214,110 +218,75 @@ class FileTests(unittest.TestCase):
         self.assertEqual(self.file.title, 'foobar')
         self.assertEqual(self.file.content_type, 'text/plain')
         self.assertTrue(ADummyCache.invalidated)
-        self.assertEquals(1, len(self.eventCatcher.modified))
+        self.assertEqual(1, len(self.eventCatcher.modified))
         self.assertTrue(self.eventCatcher.modified[0].object is self.file)
 
     def testManageUpload(self):
-        f = StringIO('jammyjohnson')
+        f = BytesIO(b'jammyjohnson')
         self.file.manage_upload(f)
-        self.assertEqual(self.file.data, 'jammyjohnson')
+        self.assertEqual(self.file.data, b'jammyjohnson')
         self.assertEqual(self.file.content_type, 'application/octet-stream')
-        self.assertEquals(1, len(self.eventCatcher.modified))
+        self.assertEqual(1, len(self.eventCatcher.modified))
         self.assertTrue(self.eventCatcher.modified[0].object is self.file)
 
+    def testManageUploadWithoutFileData(self):
+        self.file.manage_upload()
+        self.assertEqual(0, len(self.eventCatcher.modified))
+
     def testIfModSince(self):
         now = time.time()
-        e = {'SERVER_NAME':'foo', 'SERVER_PORT':'80', 'REQUEST_METHOD':'GET'}
+        e = {'SERVER_NAME': 'foo',
+             'SERVER_PORT': '80',
+             'REQUEST_METHOD': 'GET'}
 
         # not modified since
         t_notmod = rfc1123_date(now)
         e['HTTP_IF_MODIFIED_SINCE'] = t_notmod
-        out = StringIO()
+        out = BytesIO()
         resp = HTTPResponse(stdout=out)
         req = HTTPRequest(sys.stdin, e, resp)
-        data = self.file.index_html(req,resp)
+        data = self.file.index_html(req, resp)
         self.assertEqual(resp.getStatus(), 304)
-        self.assertEqual(data, '')
+        self.assertEqual(data, b'')
 
         # modified since
         t_mod = rfc1123_date(now - 100)
         e['HTTP_IF_MODIFIED_SINCE'] = t_mod
-        out = StringIO()
+        out = BytesIO()
         resp = HTTPResponse(stdout=out)
         req = HTTPRequest(sys.stdin, e, resp)
-        data = self.file.index_html(req,resp)
+        data = self.file.index_html(req, resp)
         self.assertEqual(resp.getStatus(), 200)
-        self.assertEqual(data, str(self.file.data))
-
-    def testPUT(self):
-        s = '# some python\n'
-
-        # with content type
-        data = StringIO(s)
-        req = aputrequest(data, 'text/x-python')
-        req.processInputs()
-        self.file.PUT(req, req.RESPONSE)
-
-        self.assertEqual(self.file.content_type, 'text/x-python')
-        self.assertEqual(str(self.file.data), s)
-
-        # without content type
-        data.seek(0)
-        req = aputrequest(data, '')
-        req.processInputs()
-        self.file.PUT(req, req.RESPONSE)
-
-        self.assertEqual(self.file.content_type, 'text/x-python')
-        self.assertEqual(str(self.file.data), s)
+        self.assertEqual(data, bytes(self.file.data))
 
     def testIndexHtmlWithPdata(self):
-        self.file.manage_upload('a' * (2 << 16)) # 128K
+        self.file.manage_upload(b'a' * (2 << 16))  # 128K
         self.file.index_html(self.app.REQUEST, self.app.REQUEST.RESPONSE)
-        self.assert_(self.app.REQUEST.RESPONSE._wrote)
+        self.assertTrue(self.app.REQUEST.RESPONSE._wrote)
 
     def testIndexHtmlWithString(self):
-        self.file.manage_upload('a' * 100) # 100 bytes
+        self.file.manage_upload(b'a' * 100)  # 100 bytes
         self.file.index_html(self.app.REQUEST, self.app.REQUEST.RESPONSE)
-        self.assert_(not self.app.REQUEST.RESPONSE._wrote)
+        self.assertTrue(not self.app.REQUEST.RESPONSE._wrote)
 
-    def testStr(self):
-        self.assertEqual(str(self.file), self.data)
-
-    def testFindSupport_not_text(self):
+    def testPrincipiaSearchSource_not_text(self):
+        data = ''.join([chr(x) for x in range(256)])
+        if PY3:
+            data = data.encode('utf-8')
         self.file.manage_edit('foobar', 'application/octet-stream',
-                              filedata=''.join([chr(x) for x in range(256)]))
-        self.assertEqual(self.file.PrincipiaSearchSource(), '')
+                              filedata=data)
+        self.assertEqual(self.file.PrincipiaSearchSource(), b'')
 
-    def testFindSupport_text(self):
+    def testPrincipiaSearchSource_text(self):
         self.file.manage_edit('foobar', 'text/plain',
-                              filedata='Now is the time for all good men to '
-                                       'come to the aid of the Party.')
-        self.assertTrue('Party' in self.file.PrincipiaSearchSource())
-
-    def testFindFile(self):
-        self.file.manage_edit('foobar', 'text/plain',
-                              filedata='Now is the time for all good men to '
-                                       'come to the aid of the Party.')
-        results = self.app.ZopeFind(self.app, obj_searchterm='Party')
-        self.assertEqual(len(results), 1)
-        self.assertEqual(results[0][1], self.file)
-
-    def testHistoryCompare(self):
-        self.file.manage_edit('a', 'text/plain',
-                              filedata='content_of_a')
-        getattr(self.app, self.factory)('b',
-                file='content_of_b', content_type='text/plain')
-        page = self.file.manage_historyCompare(
-            self.file,
-            self.app.b,
-            self.app.REQUEST)
-        self.assertTrue('content_of_a' in page)
-        self.assertTrue('content_of_b' in page)
+                              filedata=b'Now is the time for all good men to '
+                                       b'come to the aid of the Party.')
+        self.assertTrue(b'Party' in self.file.PrincipiaSearchSource())
 
     def test_interfaces(self):
         from zope.interface.verify import verifyClass
         from OFS.Image import File
-        from webdav.interfaces import IWriteLock
+        from OFS.interfaces import IWriteLock
         from ZPublisher.HTTPRangeSupport import HTTPRangeInterface
 
         verifyClass(HTTPRangeInterface, File)
@@ -325,12 +294,12 @@ class FileTests(unittest.TestCase):
 
     def testUnicode(self):
         val = u'some unicode string here'
-        
-        self.assertRaises(TypeError, self.file.manage_edit,
-                          'foobar', 'text/plain', filedata=val)
-        
+
+        self.assertRaises(TypeError, self.file.update_data,
+                          data=val, content_type='text/plain')
+
+
 class ImageTests(FileTests):
-    data = open(filedata, 'rb').read()
     content_type = 'image/gif'
     factory = 'manage_addImage'
 
@@ -343,45 +312,82 @@ class ImageTests(FileTests):
         self.assertTrue(ADummyCache.invalidated)
         self.assertTrue(ADummyCache.set)
 
-    def testStr(self):
-        self.assertEqual(str(self.file),
-          ('<img src="http://foo/file" alt="" title="" height="16" width="16" />'))
-
     def testTag(self):
-        tag_fmt = ('<img src="http://foo/file" alt="%s" title="%s" '
-                   'height="16" width="16" />')
-        self.assertEqual(self.file.tag(), (tag_fmt % ('','')))
+        tag_fmt = ('<img src="http://nohost/file" '
+                   'alt="%s" title="%s" height="16" width="16" />')
+        self.assertEqual(self.file.tag(), (tag_fmt % ('', '')))
         self.file.manage_changeProperties(title='foo')
-        self.assertEqual(self.file.tag(), (tag_fmt % ('','foo')))
+        self.assertEqual(self.file.tag(), (tag_fmt % ('', 'foo')))
         self.file.manage_changeProperties(alt='bar')
-        self.assertEqual(self.file.tag(), (tag_fmt % ('bar','foo')))
+        self.assertEqual(self.file.tag(), (tag_fmt % ('bar', 'foo')))
 
     def testViewImageOrFile(self):
-        pass # dtml method,screw it
+        request = self.app.REQUEST
+        response = request.RESPONSE
+        result = self.file.index_html(request, response)
+        self.assertEqual(result, self.data)
 
     def test_interfaces(self):
         from zope.interface.verify import verifyClass
         from OFS.Image import Image
-        from webdav.interfaces import IWriteLock
+        from OFS.interfaces import IWriteLock
 
         verifyClass(IWriteLock, Image)
 
-
-class ImagePublishTests(Testing.ZopeTestCase.FunctionalTestCase):
-    def testTagSafe(self):
-        self.app.manage_addImage("image", "")
-        res = self.publish(
-            "/image/tag?height=0&width=0&css_class=%22%3E%3Cscript%20type"
-            "%3D%22text%2Fjavascript%22%3Ealert('evil')%3B%3C%2Fscript"
-            "%3E%3Cdiv%20class%3D%22")
-        self.assertFalse(
-            '<script type="text/javascript">alert(\'evil\');</script>'
-                in res.getBody())
+    def test_text_representation_is_tag(self):
+        self.assertEqual(six.text_type(self.file),
+                         '<img src="http://nohost/file"'
+                         ' alt="" title="" height="16" width="16" />')
 
 
-def test_suite():
-    return unittest.TestSuite((
-        unittest.makeSuite(FileTests),
-        unittest.makeSuite(ImageTests),
-        unittest.makeSuite(ImagePublishTests)
-        ))
+class FileEditTests(Testing.ZopeTestCase.FunctionalTestCase):
+    """Browser testing ..Image.File"""
+
+    def setUp(self):
+        super(FileEditTests, self).setUp()
+        uf = self.app.acl_users
+        uf.userFolderAddUser('manager', 'manager_pass', ['Manager'], [])
+        self.app.manage_addFile('file')
+
+        transaction.commit()
+        self.browser = Testing.testbrowser.Browser()
+        self.browser.login('manager', 'manager_pass')
+
+    def test_Image__manage_main__1(self):
+        """It shows the content of text files as text."""
+        self.app.file.update_data(u'hällo'.encode('utf-8'))
+        self.browser.open('http://localhost/file/manage_main')
+        text = self.browser.getControl(name='filedata:text').value
+        self.assertEqual(text, 'hällo')
+
+    @unittest.skipIf(six.PY2, "feature not supported on Python 2")
+    def test_Image__manage_main__3(self):
+        """It shows an error message if the file content cannot be decoded."""
+        self.app.file.update_data(u'hällo'.encode('latin-1'))
+        self.browser.open('http://localhost/file/manage_main')
+        self.assertIn(
+            "The file could not be decoded with 'utf-8'.",
+            self.browser.contents)
+
+    def test_Image__manage_upload__1(self):
+        """It uploads a file, replaces the content and sets content type."""
+        self.browser.open('http://localhost/file/manage_main')
+        self.browser.getControl(name='file').add_file(
+            b'test text file', 'text/plain', 'TestFile.txt')
+        self.browser.getControl('Upload File').click()
+        self.assertIn('Saved changes', self.browser.contents)
+        self.assertEqual(
+            self.browser.getControl('Content Type').value, 'text/plain')
+        text = self.browser.getControl(name='filedata:text').value
+        self.assertEqual(text, 'test text file')
+
+    def test_Image__manage_edit__1(self):
+        """It it possible to change the file's content via browser."""
+        self.browser.open('http://localhost/file/manage_main')
+        text_1 = self.browser.getControl(name='filedata:text').value
+        self.assertEqual(text_1, '')
+        self.browser.getControl(name='filedata:text').value = u'hällo'
+        self.browser.getControl('Save Changes').click()
+        self.assertIn('Saved changes', self.browser.contents)
+        text_2 = self.browser.getControl(name='filedata:text').value
+        self.assertEqual(text_2, 'hällo')
