@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004, 2005 Zope Corporation and Contributors.
+# Copyright (c) 2004, 2005 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,25 +12,26 @@
 #
 ##############################################################################
 """A 'PageTemplateFile' without security restrictions.
-
-$Id: pagetemplatefile.py 100383 2009-05-26 01:32:41Z tseaver $
 """
-from os.path import basename
-from zope.component import getMultiAdapter
-from zope.pagetemplate.pagetemplatefile import PageTemplateFile
-from zope.pagetemplate.engine import TrustedAppPT
 
-from Acquisition import aq_get
+from os.path import basename
+
 from AccessControl import getSecurityManager
+from Acquisition import aq_get
 from Products.PageTemplates.Expressions import SecureModuleImporter
 from Products.PageTemplates.Expressions import createTrustedZopeEngine
+from zope.component import getMultiAdapter
+from zope.pagetemplate.engine import TrustedAppPT
+from zope.pagetemplate.pagetemplatefile import PageTemplateFile
 
-from Products.Five.bbb import AcquisitionBBB
 
 _engine = createTrustedZopeEngine()
+
+
 def getEngine():
     return _engine
 
+
 class ViewPageTemplateFile(TrustedAppPT, PageTemplateFile):
     """Page Template used as class variable of views defined as Python classes.
     """
@@ -47,8 +48,8 @@ class ViewPageTemplateFile(TrustedAppPT, PageTemplateFile):
 
     def __call__(self, __instance, *args, **keywords):
         # Work around BBB foul. Before Zope 2.12 there was no first argument
-        # but the Zope 3 version has one called instance. Some people used
-        # instance as an additional keyword argument.
+        # but the zope.pagetemplate version has one called instance. Some
+        # people used instance as an additional keyword argument.
         instance = __instance
         namespace = self.pt_getContext(
             request=instance.request,
@@ -58,7 +59,7 @@ class ViewPageTemplateFile(TrustedAppPT, PageTemplateFile):
             namespace,
             showtal=getattr(debug_flags, 'showTAL', 0),
             sourceAnnotations=getattr(debug_flags, 'sourceAnnotations', 0),
-            )
+        )
         response = instance.request.response
         if not response.getHeader("Content-Type"):
             response.setHeader("Content-Type", self.content_type)
@@ -88,8 +89,8 @@ class ViewPageTemplateFile(TrustedAppPT, PageTemplateFile):
                          root=root,
                          modules=SecureModuleImporter,
                          traverse_subpath=[],  # BBB, never really worked
-                         user = getSecurityManager().getUser()
-                        )
+                         user=getSecurityManager().getUser(),
+                         )
         return namespace
 
     def __get__(self, instance, type):
@@ -105,31 +106,30 @@ class ViewMapper(object):
         return getMultiAdapter((self.ob, self.request), name=name)
 
 # When a view's template is accessed e.g. as template.view, a
-# BoundPageTemplate object is retured.  For BBB reasons, it needs to
-# support the aq_* methods and attributes known from Acquisition.  For
-# that it also needs to be locatable thru __parent__.
+# BoundPageTemplate object is returned.
 
-class BoundPageTemplate(AcquisitionBBB):
+
+class BoundPageTemplate(object):
     def __init__(self, pt, ob):
-        object.__setattr__(self, 'im_func', pt)
-        object.__setattr__(self, 'im_self', ob)
+        object.__setattr__(self, '__func__', pt)
+        object.__setattr__(self, '__self__', ob)
 
-    macros = property(lambda self: self.im_func.macros)
-    filename = property(lambda self: self.im_func.filename)
-    __parent__ = property(lambda self: self.im_self)
+    macros = property(lambda self: self.__func__.macros)
+    filename = property(lambda self: self.__func__.filename)
+    __parent__ = property(lambda self: self.__self__)
 
     def __call__(self, *args, **kw):
-        if self.im_self is None:
-            im_self, args = args[0], args[1:]
+        if self.__self__ is None:
+            __self__, args = args[0], args[1:]
         else:
-            im_self = self.im_self
-        return self.im_func(im_self, *args, **kw)
+            __self__ = self.__self__
+        return self.__func__(__self__, *args, **kw)
 
     def __setattr__(self, name, v):
         raise AttributeError("Can't set attribute", name)
 
     def __repr__(self):
-        return "<BoundPageTemplateFile of %r>" % self.im_self
+        return "<BoundPageTemplateFile of %r>" % self.__self__
 
 
 # BBB
