@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2004, 2005 Zope Corporation and Contributors.
+# Copyright (c) 2004, 2005 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,10 +12,9 @@
 #
 ##############################################################################
 """Test Five-traversable classes
-
-$Id: test_traversable.py 99527 2009-04-26 16:33:20Z hannosch $
 """
 
+
 class SimpleClass(object):
     """Class with no __bobo_traverse__."""
 
@@ -25,8 +24,10 @@ def test_traversable():
     Test the behaviour of Five-traversable classes.
 
       >>> import Products.Five
-      >>> from Products.Five import zcml
+      >>> from Zope2.App import zcml
       >>> zcml.load_config("configure.zcml", Products.Five)
+      >>> folder = self.folder  # NOQA: F821
+      >>> http = http  # NOQA: F821
 
     ``SimpleContent`` is a traversable class by default.  Its fallback
     traverser should raise NotFound when traversal fails.  (Note: If
@@ -34,11 +35,12 @@ def test_traversable():
     the wrong reason: None doesn't have a docstring so BaseRequest
     raises NotFoundError.)
 
-      >>> from Products.Five.tests.testing.simplecontent import manage_addSimpleContent
-      >>> manage_addSimpleContent(self.folder, 'testoid', 'Testoid')
-      >>> print http(r'''
+      >>> from Products.Five.tests.testing.simplecontent import (
+      ... manage_addSimpleContent)
+      >>> manage_addSimpleContent(folder, 'testoid', 'Testoid')
+      >>> print(http(r'''
       ... GET /test_folder_1_/testoid/doesntexist HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 404 Not Found
       ...
 
@@ -50,7 +52,7 @@ def test_traversable():
       ...            xmlns:meta="http://namespaces.zope.org/meta"
       ...            xmlns:browser="http://namespaces.zope.org/browser"
       ...            xmlns:five="http://namespaces.zope.org/five">
-      ... 
+      ...
       ... <!-- make the zope2.Public permission work -->
       ... <meta:redefinePermission from="zope2.Public" to="zope.Public" />
       ...
@@ -80,15 +82,16 @@ def test_traversable():
       ... </configure>'''
       >>> zcml.load_string(configure_zcml)
 
-      >>> from Products.Five.tests.testing.fancycontent import manage_addFancyContent
-      >>> info = manage_addFancyContent(self.folder, 'fancy', '')
+      >>> from Products.Five.tests.testing.fancycontent import (
+      ... manage_addFancyContent)
+      >>> info = manage_addFancyContent(folder, 'fancy', '')
 
     In the following test we let the original __bobo_traverse__ method
     kick in:
 
-      >>> print http(r'''
+      >>> print(http(r'''
       ... GET /test_folder_1_/fancy/something-else HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       something-else
@@ -97,9 +100,9 @@ def test_traversable():
     takes over.  Therefore, unless it raises AttributeError or
     KeyError, it will be the only way traversal is done.
 
-      >>> print http(r'''
+      >>> print(http(r'''
       ... GET /test_folder_1_/fancy/fancyview HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       fancyview
@@ -108,40 +111,42 @@ def test_traversable():
     AttributeError or KeyError, we can get normal view look-up.  Other
     exceptions are passed through just fine:
 
-      >>> print http(r'''
+      >>> print(http(r'''
       ... GET /test_folder_1_/fancy/raise-attributeerror HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       Fancy, fancy
 
-      >>> print http(r'''
+      >>> print(http(r'''
       ... GET /test_folder_1_/fancy/raise-keyerror HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       Fancy, fancy
 
-      >>> print http(r'''
+      >>> print(http(r'''
       ... GET /test_folder_1_/fancy/raise-valueerror HTTP/1.1
-      ... ''', handle_errors=False)
+      ... ''', handle_errors=False))
       Traceback (most recent call last):
         ...
       ValueError: ...
 
     Five's traversable monkeypatches the __bobo_traverse__ method to do view
     lookup and then delegates back to the original __bobo_traverse__ or direct
-    attribute/item lookup to do normal lookup.  In the Zope 2 ZPublisher, an 
+    attribute/item lookup to do normal lookup.  In the Zope 2 ZPublisher, an
     object with a __bobo_traverse__ will not do attribute lookup unless the
     __bobo_traverse__ method itself does it (i.e. the __bobo_traverse__ is the
     only element used for traversal lookup).  Let's demonstrate:
 
-      >>> from Products.Five.tests.testing.fancycontent import manage_addNonTraversableFancyContent
-      >>> info = manage_addNonTraversableFancyContent(self.folder, 'fancy_zope2', '')
-      >>> self.folder.fancy_zope2.an_attribute = 'This is an attribute'
-      >>> print http(r'''
+      >>> from Products.Five.tests.testing.fancycontent import (
+      ... manage_addNonTraversableFancyContent)
+      >>> info = manage_addNonTraversableFancyContent(
+      ...     folder, 'fancy_zope2', '')
+      >>> folder.fancy_zope2.an_attribute = 'This is an attribute'
+      >>> print(http(r'''
       ... GET /test_folder_1_/fancy_zope2/an_attribute HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       an_attribute
@@ -150,10 +155,10 @@ def test_traversable():
     value 'This is an attribute'.  Let's make sure the same thing happens for
     an object that has been marked traversable by Five:
 
-      >>> self.folder.fancy.an_attribute = 'This is an attribute'
-      >>> print http(r'''
+      >>> folder.fancy.an_attribute = 'This is an attribute'
+      >>> print(http(r'''
       ... GET /test_folder_1_/fancy/an_attribute HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       an_attribute
@@ -181,6 +186,7 @@ def test_traversable():
       False
     """
 
+
 def test_view_doesnt_shadow_attribute():
     """
     Test that views don't shadow attributes, e.g. items in a folder.
@@ -211,35 +217,39 @@ def test_view_doesnt_shadow_attribute():
       ...       />
       ... </configure>'''
       >>> import Products.Five
-      >>> from Products.Five import zcml
+      >>> from Zope2.App import zcml
       >>> zcml.load_config("configure.zcml", Products.Five)
       >>> zcml.load_string(configure_zcml)
+      >>> folder = self.folder  # NOQA: F821
+      >>> http = http  # NOQA: F821
 
     Then we create a traversable folder...
 
-      >>> from Products.Five.tests.testing.folder import manage_addFiveTraversableFolder
-      >>> manage_addFiveTraversableFolder(self.folder, 'ftf')
+      >>> from Products.Five.tests.testing.folder import (
+      ... manage_addFiveTraversableFolder)
+      >>> manage_addFiveTraversableFolder(folder, 'ftf')
 
     and add an object called ``eagle`` to it:
 
-      >>> from Products.Five.tests.testing.simplecontent import manage_addIndexSimpleContent
-      >>> manage_addIndexSimpleContent(self.folder.ftf, 'eagle', 'Eagle')
+      >>> from Products.Five.tests.testing.simplecontent import (
+      ... manage_addIndexSimpleContent)
+      >>> manage_addIndexSimpleContent(folder.ftf, 'eagle', 'Eagle')
 
     When we publish the ``ftf/eagle`` now, we expect the attribute to
     take precedence over the view during traversal:
 
-      >>> print http(r'''
+      >>> print(http(r'''
       ... GET /test_folder_1_/ftf/eagle HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       Default index_html called
 
     Of course, unless we explicitly want to lookup the view using @@:
 
-      >>> print http(r'''
+      >>> print(http(r'''
       ... GET /test_folder_1_/ftf/@@eagle HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       The eagle has landed
@@ -249,39 +259,40 @@ def test_view_doesnt_shadow_attribute():
     found in OFS.Application, raise NotFound.  Five still knows how to
     deal with this, hence views work there too:
 
-      >>> print http(r'''
+      >>> print(http(r'''
       ... GET /eagle HTTP/1.1
       ...
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       The eagle has landed
 
-      >>> print http(r'''
+      >>> print(http(r'''
       ... GET /@@eagle HTTP/1.1
       ...
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       The eagle has landed
 
     However, acquired attributes *should* be shadowed. See discussion on
     http://codespeak.net/pipermail/z3-five/2006q2/001474.html
-    
-      >>> manage_addIndexSimpleContent(self.folder, 'mouse', 'Mouse')
-      >>> print http(r'''
+
+      >>> manage_addIndexSimpleContent(folder, 'mouse', 'Mouse')
+      >>> print(http(r'''
       ... GET /test_folder_1_/ftf/mouse HTTP/1.1
-      ... ''')
+      ... '''))
       HTTP/1.1 200 OK
       ...
       The mouse has been eaten by the eagle
-      
+
     Clean up:
 
       >>> from zope.component.testing import tearDown
       >>> tearDown()
     """
 
+
 def test_suite():
     from Testing.ZopeTestCase import FunctionalDocTestSuite
     return FunctionalDocTestSuite()
