@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Foundation and Contributors.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -14,18 +14,28 @@
 
 Page Template-specific implementation of TALES, with handlers
 for Python expressions, string literals, and paths.
-
-$Id: Expressions.py 107725 2010-01-06 01:31:21Z lra $
 """
 
 import logging
 
+from six import binary_type
+from six import text_type
+
+import OFS.interfaces
+from Acquisition import aq_base
+from MultiMapping import MultiMapping
+from Products.PageTemplates import ZRPythonExpr
+from Products.PageTemplates.interfaces import IUnicodeEncodingConflictResolver
+from zExceptions import NotFound
+from zExceptions import Unauthorized
 from zope.component import queryUtility
+from zope.contentprovider.tales import TALESProviderExpression
 from zope.i18n import translate
-from zope.interface import implements
+from zope.interface import implementer
 from zope.pagetemplate.engine import ZopeEngine as Z3Engine
 from zope.proxy import removeAllProxies
 from zope.tales.expressions import DeferExpr
+from zope.tales.expressions import LazyExpr
 from zope.tales.expressions import NotExpr
 from zope.tales.expressions import PathExpr
 from zope.tales.expressions import StringExpr
@@ -34,35 +44,26 @@ from zope.tales.pythonexpr import PythonExpr
 from zope.tales.tales import Context
 from zope.tales.tales import ErrorInfo as BaseErrorInfo
 from zope.tales.tales import Iterator
-from zope.traversing.interfaces import ITraversable
 from zope.traversing.adapters import traversePathElement
+from zope.traversing.interfaces import ITraversable
 
-import OFS.interfaces
-from MultiMapping import MultiMapping
-from Acquisition import aq_base
-from zExceptions import NotFound
-from zExceptions import Unauthorized
-
-from zope.contentprovider.tales import TALESProviderExpression
-from Products.PageTemplates import ZRPythonExpr
-from Products.PageTemplates.DeferExpr import LazyExpr
-from Products.PageTemplates.interfaces import IUnicodeEncodingConflictResolver
 
 SecureModuleImporter = ZRPythonExpr._SecureModuleImporter()
 
 LOG = logging.getLogger('Expressions')
 
 # In Zope 2 traversal semantics, NotFound or Unauthorized (the Zope 2
-# versions) indicate that traversal has failed.  By default, Zope 3's
-# TALES engine doesn't recognize them as such which is why we extend
-# Zope 3's list here and make sure our implementation of the TALES
+# versions) indicate that traversal has failed.  By default, zope.tales'
+# engine doesn't recognize them as such which is why we extend its
+# list here and make sure our implementation of the TALES
 # Path Expression uses them
 ZopeUndefs = Undefs + (NotFound, Unauthorized)
 
+
 def boboAwareZopeTraverse(object, path_items, econtext):
     """Traverses a sequence of names, first trying attributes then items.
 
-    This uses Zope 3 path traversal where possible and interacts
+    This uses zope.traversing path traversal where possible and interacts
     correctly with objects providing OFS.interface.ITraversable when
     necessary (bobo-awareness).
     """
@@ -79,10 +80,11 @@ def boboAwareZopeTraverse(object, path_items, econtext):
                                          request=request)
     return object
 
+
 def trustedBoboAwareZopeTraverse(object, path_items, econtext):
     """Traverses a sequence of names, first trying attributes then items.
 
-    This uses Zope 3 path traversal where possible and interacts
+    This uses zope.traversing path traversal where possible and interacts
     correctly with objects providing OFS.interface.ITraversable when
     necessary (bobo-awareness).
     """
@@ -99,6 +101,7 @@ def trustedBoboAwareZopeTraverse(object, path_items, econtext):
                                          request=request)
     return object
 
+
 def render(ob, ns):
     """Calls the object, possibly a document template, or just returns
     it if not callable.  (From DT_Util.py)
@@ -117,11 +120,11 @@ def render(ob, ns):
                     ob = ZRPythonExpr.call_with_ns(ob, ns, 2)
                 else:
                     ob = ob()
-            except AttributeError, n:
-                if str(n) != '__call__':
-                    raise
+            except NotImplementedError:
+                pass
     return ob
 
+
 class ZopePathExpr(PathExpr):
 
     _TRAVERSER = staticmethod(boboAwareZopeTraverse)
@@ -140,7 +143,7 @@ class ZopePathExpr(PathExpr):
             # Try all but the last subexpression, skipping undefined ones.
             try:
                 ob = expr(econtext)
-            except ZopeUndefs: # use Zope 2 expression types
+            except ZopeUndefs:  # use Zope 2 expression types
                 pass
             else:
                 break
@@ -162,15 +165,17 @@ class ZopePathExpr(PathExpr):
         for expr in self._subexprs:
             try:
                 expr(econtext)
-            except ZopeUndefs: # use Zope 2 expression types
+            except ZopeUndefs:  # use Zope 2 expression types
                 pass
             else:
                 return 1
         return 0
 
+
 class TrustedZopePathExpr(ZopePathExpr):
     _TRAVERSER = staticmethod(trustedBoboAwareZopeTraverse)
 
+
 class SafeMapping(MultiMapping):
     """Mapping with security declarations and limited method exposure.
 
@@ -185,6 +190,7 @@ class SafeMapping(MultiMapping):
     _push = MultiMapping.push
     _pop = MultiMapping.pop
 
+
 class ZopeContext(Context):
 
     def __init__(self, engine, contexts):
@@ -205,7 +211,7 @@ class ZopeContext(Context):
 
     def evaluateBoolean(self, expr):
         value = self.evaluate(expr)
-        # here we override the normal Zope 3 behaviour.  Zope 3
+        # here we override the normal zope.tales behaviour. zope.tales
         # doesn't care about the default in a boolean expression,
         # while we do (Zope 2 legacy, see the
         # BooleanAttributesAndDefault.html test case)
@@ -233,11 +239,11 @@ class ZopeContext(Context):
             # XXX: should be unicode???
             return text
 
-        if isinstance(text, unicode):
+        if isinstance(text, text_type):
             # we love unicode, nothing to do
             return text
 
-        elif isinstance(text, str):
+        elif isinstance(text, binary_type):
             # bahh...non-unicode string..we need to convert it to unicode
 
             # catch ComponentLookupError in order to make tests shut-up.
@@ -246,24 +252,23 @@ class ZopeContext(Context):
 
             resolver = queryUtility(IUnicodeEncodingConflictResolver)
             if resolver is None:
-                return unicode(text)
+                return text.decode('ascii')
 
             try:
                 return resolver.resolve(self.contexts['context'], text, expr)
-            except UnicodeDecodeError,e:
-                LOG.error("""UnicodeDecodeError detected for expression "%s"\n"""
-                          """Resolver class: %s\n"""
-                          """Exception text: %s\n"""
-                          """Template: %s\n"""
-                          """Rendered text: %r"""  % \
-                          (expr, resolver.__class__, e, 
-                            self.contexts['template'].absolute_url(1), text))
-                raise 
+            except UnicodeDecodeError as e:
+                LOG.error("UnicodeDecodeError detected for expression \"%s\"\n"
+                          "Resolver class: %s\n"
+                          "Exception text: %s\n"
+                          "Template: %s\n"
+                          "Rendered text: %r" %
+                          (expr, resolver.__class__, e,
+                           self.contexts['template'].absolute_url(1), text))
+                raise
         else:
-
-            # This is a weird culprit ...calling unicode() on non-string
+            # This is a weird culprit ...calling text_type() on non-string
             # objects
-            return unicode(text)
+            return text_type(text)
 
     def createErrorInfo(self, err, position):
         # Override, returning an object accessible to untrusted code.
@@ -278,15 +283,18 @@ class ZopeContext(Context):
         """
         raise NotImplementedError
 
+
 class ErrorInfo(BaseErrorInfo):
     """Information about an exception passed to an on-error handler.
     """
     __allow_access_to_unprotected_subobjects__ = True
 
+
 class ZopeEngine(Z3Engine):
 
     _create_context = ZopeContext
 
+
 class ZopeIterator(Iterator):
 
     # allow iterator API to be accessed from (restricted) Python TALES
@@ -317,7 +325,7 @@ class ZopeIterator(Iterator):
         return super(ZopeIterator, self).item()
 
     # 'first' and 'last' are Zope 2 enhancements to the TALES iterator
-    # spec.  See help/tal-repeat.stx for more info
+    # spec.
     def first(self, name=None):
         if self.start:
             return True
@@ -335,17 +343,21 @@ class ZopeIterator(Iterator):
         return getattr(ob1, name, no) == getattr(ob2, name, no) is not no
 
     # 'first' needs to have access to the last item in the loop
+    def __next__(self):
+        if self._nextIndex > 0:
+            self._last_item = self.item
+        return super(ZopeIterator, self).__next__()
+
     def next(self):
         if self._nextIndex > 0:
             self._last_item = self.item
         return super(ZopeIterator, self).next()
 
+
+@implementer(ITraversable)
 class PathIterator(ZopeIterator):
     """A TALES Iterator with the ability to use first() and last() on
     subpaths of elements."""
-    # we want to control our own traversal so that we can deal with
-    # 'first' and 'last' when they appear in path expressions
-    implements(ITraversable)
 
     def traverse(self, name, furtherPath):
         if name in ('first', 'last'):
@@ -363,8 +375,10 @@ class PathIterator(ZopeIterator):
     def same_part(self, name, ob1, ob2):
         if name is None:
             return ob1 == ob2
-        if isinstance(name, basestring):
-            name = name.split('/')
+        if isinstance(name, text_type):
+            name = name.split(u'/')
+        elif isinstance(name, binary_type):
+            name = name.split(b'/')
         try:
             ob1 = boboAwareZopeTraverse(ob1, name, None)
             ob2 = boboAwareZopeTraverse(ob2, name, None)
@@ -372,11 +386,12 @@ class PathIterator(ZopeIterator):
             return False
         return ob1 == ob2
 
+
 class UnicodeAwareStringExpr(StringExpr):
 
     def __call__(self, econtext):
         vvals = []
-        if isinstance(self._expr, unicode):
+        if isinstance(self._expr, text_type):
             # coerce values through the Unicode Conflict Resolver
             evaluate = econtext.evaluateText
         else:
@@ -386,6 +401,7 @@ class UnicodeAwareStringExpr(StringExpr):
             vvals.append(v)
         return self._expr % tuple(vvals)
 
+
 def createZopeEngine(zpe=ZopePathExpr):
     e = ZopeEngine()
     e.iteratorFactory = PathIterator
@@ -400,6 +416,7 @@ def createZopeEngine(zpe=ZopePathExpr):
     e.registerBaseName('modules', SecureModuleImporter)
     return e
 
+
 def createTrustedZopeEngine():
     # same as createZopeEngine, but use non-restricted Python
     # expression evaluator
@@ -407,6 +424,9 @@ def createTrustedZopeEngine():
     e.types['python'] = PythonExpr
     return e
 
+
 _engine = createZopeEngine()
+
+
 def getEngine():
     return _engine
