@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Foundation and Contributors.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -13,33 +13,33 @@
 """Zope-specific Python Expression Handler
 
 Handler for Python expressions that uses the RestrictedPython package.
-
-$Id: ZRPythonExpr.py 106134 2009-11-30 23:21:51Z tseaver $
 """
+
 from AccessControl import safe_builtins
-from AccessControl.ZopeGuards import guarded_getattr, get_safe_globals
+from AccessControl.ZopeGuards import get_safe_globals
+from AccessControl.ZopeGuards import guarded_getattr
+from DocumentTemplate.DT_Util import InstanceDict
+from DocumentTemplate.DT_Util import TemplateDict
+from DocumentTemplate.security import RestrictedDTML
 from RestrictedPython import compile_restricted_eval
-from zope.tales.tales import CompilerError
 from zope.tales.pythonexpr import PythonExpr
 
+
 class PythonExpr(PythonExpr):
     _globals = get_safe_globals()
     _globals['_getattr_'] = guarded_getattr
-    _globals['__debug__' ] = __debug__
+    _globals['__debug__'] = __debug__
 
     def __init__(self, name, expr, engine):
         self.text = self.expr = text = expr.strip().replace('\n', ' ')
+        code, err, warn, use = compile_restricted_eval(
+            text, self.__class__.__name__)
 
-        # Unicode expression are not handled properly by RestrictedPython
-        # We convert the expression to UTF-8 (ajung)
-        if isinstance(text, unicode):
-            text = text.encode('utf-8')
-        code, err, warn, use = compile_restricted_eval(text, 
-                                                       self.__class__.__name__)
         if err:
-            raise engine.getCompilerError()('Python expression error:\n%s' %
-                                            '\n'.join(err))            
-        self._varnames = use.keys()
+            raise engine.getCompilerError()(
+                'Python expression error:\n%s' % '\n'.join(err))
+
+        self._varnames = list(use.keys())
         self._code = code
 
     def __call__(self, econtext):
@@ -48,7 +48,8 @@ class PythonExpr(PythonExpr):
         vars.update(self._globals)
         return eval(self._code, vars, {})
 
-class _SecureModuleImporter:
+
+class _SecureModuleImporter(object):
     __allow_access_to_unprotected_subobjects__ = True
 
     def __getitem__(self, module):
@@ -58,11 +59,11 @@ class _SecureModuleImporter:
             mod = getattr(mod, name)
         return mod
 
-from DocumentTemplate.DT_Util import TemplateDict, InstanceDict
-from AccessControl.DTML import RestrictedDTML
+
 class Rtd(RestrictedDTML, TemplateDict):
     this = None
 
+
 def call_with_ns(f, ns, arg=1):
     td = Rtd()
     # prefer 'context' to 'here';  fall back to 'None'
@@ -75,7 +76,7 @@ def call_with_ns(f, ns, arg=1):
     td._push(InstanceDict(td.this, td))
     td._push(ns)
     try:
-        if arg==2:
+        if arg == 2:
             return f(None, td)
         else:
             return f(td)
