@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Foundation and Contributors.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -11,39 +11,42 @@
 #
 ##############################################################################
 
-__version__='$Revision$'[11:-2]
-
 import re
-import string
 
+from six import exec_
+
+from AccessControl.class_init import InitializeClass
+from AccessControl.PermissionRole import _what_not_even_god_should_do
+from AccessControl.Permissions import view_management_screens
 from AccessControl.SecurityInfo import ClassSecurityInfo
 from AccessControl.SecurityManagement import getSecurityManager
-from AccessControl.Permissions import view_management_screens
-from AccessControl.PermissionRole import _what_not_even_god_should_do
+from AccessControl.unauthorized import Unauthorized
 from AccessControl.ZopeGuards import guarded_getattr
-from Acquisition import aq_parent
+from Acquisition import aq_base
 from Acquisition import aq_inner
-from App.class_init import InitializeClass
-from Persistence import Persistent
+from Acquisition import aq_parent
+from zope.component import queryMultiAdapter as qma
+
 
 defaultBindings = {'name_context': 'context',
                    'name_container': 'container',
                    'name_m_self': 'script',
                    'name_ns': '',
                    'name_subpath': 'traverse_subpath',
-                  }
+                   }
 
 _marker = []  # Create a new marker
 
-class NameAssignments:
+
+class NameAssignments(object):
     # Note that instances of this class are intended to be immutable
     # and persistent but not inherit from ExtensionClass.
 
-    _exprs = (('name_context',   'self._getContext()'),
+    _exprs = (('name_context', 'self._getContext()'),
               ('name_container', 'self._getContainer()'),
-              ('name_m_self',    'self'),
-              ('name_ns',        'self._getNamespace(caller_namespace, kw)'),
-              ('name_subpath',   'self._getTraverseSubpath()'),
+              ('name_m_self', 'self'),
+              ('name_ns', 'self._getNamespace(caller_namespace, kw)'),
+              ('name_subpath', 'self._getTraverseSubpath()'),
               )
 
     _isLegalName = re.compile(r'_$|[a-zA-Z][a-zA-Z0-9_]*$').match
@@ -51,19 +54,26 @@ class NameAssignments:
 
     __allow_access_to_unprotected_subobjects__ = 1
 
-    def __init__(self, mapping):
+    def __init__(self, mapping=None):
         # mapping is presumably the REQUEST or compatible equivalent.
         # Note that we take care not to store expression texts in the ZODB.
+
+        # The default value is needed for unpickling instances of this class
+        # which where created before 4.0b2 where this class was still an old
+        # style class. For details see
+        # https://github.com/zopefoundation/Zope/issues/205
         asgns = {}
+        if mapping is None:
+            mapping = {}
         _isLegalName = self._isLegalName
         for name, expr in self._exprs:
-            if mapping.has_key(name):
-                assigned_name = string.strip(mapping[name])
+            if name in mapping:
+                assigned_name = mapping[name].strip()
                 if not assigned_name:
                     continue
                 if not _isLegalName(assigned_name):
-                    raise ValueError, ('"%s" is not a valid variable name.'
-                                       % assigned_name)
+                    raise ValueError('"%s" is not a valid variable name.' %
+                                     assigned_name)
                 asgns[name] = assigned_name
         self._asgns = asgns
 
@@ -73,12 +83,12 @@ class NameAssignments:
         return 0
 
     def isNameAssigned(self, name):
-        return self._asgns.has_key(name)
+        return name in self._asgns
 
     def getAssignedName(self, name, default=_marker):
         val = self._asgns.get(name, default)
         if val is _marker:
-            raise KeyError, name
+            raise KeyError(name)
         return val
 
     def getAssignedNames(self):
@@ -91,7 +101,7 @@ class NameAssignments:
         rval = []
         asgns = self._asgns
         for name, expr in self._exprs:
-            if asgns.has_key(name):
+            if name in asgns:
                 assigned_name = asgns[name]
                 rval.append(assigned_name)
         return rval
@@ -102,9 +112,9 @@ class NameAssignments:
         # and the number of names bound.
         text = ['bound_data.append(%s)\n' % bindtext]
         for assigned_name in assigned_names:
-            text.append('if kw.has_key("%s"):\n' % assigned_name)
-            text.append('    del kw["%s"]\n'     % assigned_name)
-        codetext = string.join(text, '')
+            text.append('if "%s" in kw:\n' % assigned_name)
+            text.append('    del kw["%s"]\n' % assigned_name)
+        codetext = ''.join(text)
         return (compile(codetext, '<string>', 'exec'), len(assigned_names))
 
     def _createCodeBlockForMapping(self):
@@ -116,11 +126,11 @@ class NameAssignments:
         assigned_names = []
         asgns = self._asgns
         for name, expr in self._exprs:
-            if asgns.has_key(name):
+            if name in asgns:
                 assigned_name = asgns[name]
                 assigned_names.append(assigned_name)
                 exprtext.append('"%s":%s,' % (assigned_name, expr))
-        text = '{%s}' % string.join(exprtext, '')
+        text = '{%s}' % ''.join(exprtext)
         return self._generateCodeBlock(text, assigned_names)
 
     def _createCodeBlockForTuple(self, argNames):
@@ -135,7 +145,7 @@ class NameAssignments:
             passedLastBoundArg = 1
             for name, expr in self._exprs:
                 # Provide a value for the available exprs.
-                if asgns.has_key(name):
+                if name in asgns:
                     assigned_name = asgns[name]
                     if assigned_name == argName:
                         # The value for this argument will be filled in.
@@ -146,13 +156,11 @@ class NameAssignments:
             if passedLastBoundArg:
                 # Found last of bound args.
                 break
-        text = '(%s)' % string.join(exprtext, '')
+        text = '(%s)' % ''.join(exprtext)
         return self._generateCodeBlock(text, assigned_names)
 
 
-from AccessControl.unauthorized import Unauthorized
-
-class UnauthorizedBinding:
+class UnauthorizedBinding(object):
     """Explanation: as of Zope 2.6.3 a security hole was closed - no
        security check was happening when 'context' and 'container'
        were bound to a script. Adding the check broke lots of sites
@@ -193,7 +201,6 @@ class UnauthorizedBinding:
             self.__you_lose()
 
         return guarded_getattr(self._wrapped, name, default)
-        #return getattr(self._wrapped, name, default)
 
     def __you_lose(self):
         name = self.__dict__['_name']
@@ -201,19 +208,20 @@ class UnauthorizedBinding:
 
     __str__ = __call__ = index_html = __you_lose
 
-class Bindings:
+
+class Bindings(object):
 
     security = ClassSecurityInfo()
 
     _Bindings_client = None
 
-    security.declareProtected('Change bindings', 'ZBindings_edit')
+    @security.protected('Change bindings')
     def ZBindings_edit(self, mapping):
-        names = self._setupBindings(mapping)
+        self._setupBindings(mapping)
         self._prepareBindCode()
         self._editedBindings()
 
-    security.declareProtected('Change bindings', 'ZBindings_setClient')
+    @security.protected('Change bindings')
     def ZBindings_setClient(self, clientname):
         '''Name the binding to be used as the "client".
 
@@ -229,7 +237,7 @@ class Bindings:
         self._bind_names = names = NameAssignments(names)
         return names
 
-    security.declareProtected(view_management_screens, 'getBindingAssignments')
+    @security.protected(view_management_screens)
     def getBindingAssignments(self):
         if not hasattr(self, '_bind_names'):
             self._setupBindings()
@@ -238,8 +246,9 @@ class Bindings:
     def __before_publishing_traverse__(self, self2, request):
         path = request['TraversalRequestNameStack']
         names = self.getBindingAssignments()
-        if (not names.isNameAssigned('name_subpath') or
-            (path and hasattr(self.aq_base, path[-1])) ):
+        if not names.isNameAssigned('name_subpath') or \
+           (path and hasattr(aq_base(self), path[-1])) or \
+           (path and qma((self, request), name=path[-1]) is not None):
             return
         subpath = path[:]
         path[:] = []
@@ -280,7 +289,8 @@ class Bindings:
                 parent = aq_parent(self)
                 inner = aq_inner(self)
                 container = aq_parent(inner)
-                try: getSecurityManager().validate(parent, container, '', self)
+                try:
+                    getSecurityManager().validate(parent, container, '', self)
                 except Unauthorized:
                     return UnauthorizedBinding('context', self)
                 return self
@@ -293,7 +303,8 @@ class Bindings:
                 parent = aq_parent(self)
                 inner = aq_inner(self)
                 container = aq_parent(inner)
-                try: getSecurityManager().validate(parent, container, '', self)
+                try:
+                    getSecurityManager().validate(parent, container, '', self)
                 except Unauthorized:
                     return UnauthorizedBinding('container', self)
                 return self
@@ -329,7 +340,7 @@ class Bindings:
         # Try to find unbound parameters in the namespace, if the
         # namespace is bound.
         if self.getBindingAssignments().isNameAssigned('name_ns'):
-            code = self.func_code
+            code = self.__code__
             for name in code.co_varnames[:code.co_argcount]:
                 try:
                     namevals[name] = namespace[name]
@@ -356,10 +367,11 @@ class Bindings:
                 bound_data = {}
             else:
                 bound_data = []
-                exec bindcode
+                exec_(bindcode)
                 bound_data = bound_data[0]
             return self._exec(bound_data, args, kw)
         finally:
             security.removeContext(self)
 
+
 InitializeClass(Bindings)
