@@ -1,13 +1,13 @@
 ##############################################################################
 #
-# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Foundation and Contributors.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """XML-RPC support module
@@ -21,18 +21,23 @@ information about XML-RPC and Zope.
 
 import re
 import sys
-import types
-import xmlrpclib
-
-from zExceptions import Unauthorized
-from ZODB.POSException import ConflictError
-
-from ZPublisher.HTTPResponse import HTTPResponse
 
+from App.config import getConfiguration
 # Make DateTime.DateTime marshallable via XML-RPC
 # http://www.zope.org/Collectors/Zope/2109
 from DateTime.DateTime import DateTime
-WRAPPERS = xmlrpclib.WRAPPERS + (DateTime,)
+from zExceptions import Unauthorized
+from ZODB.POSException import ConflictError
+
+
+try:
+    import xmlrpc.client as xmlrpclib
+except ImportError:
+    import xmlrpclib
+
+
+WRAPPERS = xmlrpclib.WRAPPERS + (DateTime, )
+
 
 def dump_instance(self, value, write):
     # Check for special wrappers
@@ -49,7 +54,17 @@ def dump_instance(self, value, write):
                       if k[:1] != '_'])
         self.dump_struct(value, write)
 
-xmlrpclib.Marshaller.dispatch[types.InstanceType] = dump_instance
+
+# Override the standard marshaller for object instances
+# to skip private attributes.
+try:
+    from types import InstanceType
+    xmlrpclib.Marshaller.dispatch[InstanceType] = dump_instance  # py2
+except ImportError:
+    xmlrpclib.Marshaller.dispatch['_arbitrary_instance'] = dump_instance  # py3
+
+xmlrpclib.Marshaller.dispatch[DateTime] = dump_instance
+
 
 def parse_input(data):
     """Parse input data and return a method path and argument tuple
@@ -94,10 +109,15 @@ def parse_input(data):
 ########################################################################
 # Possible implementation helpers:
 
-class Response:
+
+def is_xmlrpc_response(response):
+    return isinstance(response, Response)
+
+
+class Response(object):
     """Customized Response that handles XML-RPC-specific details.
 
-    We override setBody to marhsall Python objects into XML-RPC. We
+    We override setBody to marshall Python objects into XML-RPC. We
     also override exception to convert errors to XML-RPC faults.
 
     If these methods stop getting called, make sure that ZPublisher is
@@ -107,23 +127,29 @@ class Response:
     It's probably possible to improve the 'exception' method quite a bit.
     The current implementation, however, should suffice for now.
     """
-    
-    _error_format = 'text/plain' # No html in error values
+
+    _error_format = 'text/plain'  # No html in error values
 
     # Because we can't predict what kind of thing we're customizing,
     # we have to use delegation, rather than inheritance to do the
     # customization.
 
-    def __init__(self, real): self.__dict__['_real']=real
+    def __init__(self, real):
+        self.__dict__['_real'] = real
 
-    def __getattr__(self, name): return getattr(self._real, name)
-    def __setattr__(self, name, v): return setattr(self._real, name, v)
-    def __delattr__(self, name): return delattr(self._real, name)
+    def __getattr__(self, name):
+        return getattr(self._real, name)
+
+    def __setattr__(self, name, v):
+        return setattr(self._real, name, v)
+
+    def __delattr__(self, name):
+        return delattr(self._real, name)
 
     def setBody(self, body, title='', is_error=0, bogus_str_search=None):
         if isinstance(body, xmlrpclib.Fault):
             # Convert Fault object to XML-RPC response.
-            body=xmlrpclib.dumps(body, methodresponse=1, allow_none=True)
+            body = xmlrpclib.dumps(body, methodresponse=1, allow_none=True)
         else:
             # Marshall our body as an XML-RPC response. Strings will be sent
             # strings, integers as integers, etc. We do *not* convert
@@ -136,7 +162,7 @@ class Response:
                     (body,), methodresponse=1, allow_none=True)
             except ConflictError:
                 raise
-            except:
+            except Exception:
                 self.exception()
                 return
         # Set our body to the XML-RPC message, and fix our MIME type.
@@ -148,44 +174,41 @@ class Response:
                   absuri_match=None, tag_search=None):
         # Fetch our exception info. t is type, v is value and tb is the
         # traceback object.
-        if type(info) is type(()) and len(info)==3: t,v,tb = info
-        else: t,v,tb = sys.exc_info()
+        if isinstance(info, tuple) and len(info) == 3:
+            t, v, tb = info
+        else:
+            t, v, tb = sys.exc_info()
 
-        # Don't mask 404 respnses, as some XML-RPC libraries rely on the HTTP
+        # Don't mask 404 responses, as some XML-RPC libraries rely on the HTTP
         # mechanisms for detecting when authentication is required. Fixes Zope
         # Collector issue 525.
-        if t == 'Unauthorized' or (
-            isinstance(t, types.ClassType) and issubclass(t, Unauthorized)
-            ):
-
+        if issubclass(t, Unauthorized):
             return self._real.exception(fatal=fatal, info=info)
 
         # Create an appropriate Fault object. Containing error information
-        Fault=xmlrpclib.Fault
-        f=None
+        Fault = xmlrpclib.Fault
+        f = None
         try:
             # Strip HTML tags from the error value
             vstr = str(v)
             remove = [r"<[^<>]*>", r"&[A-Za-z]+;"]
             for pat in remove:
                 vstr = re.sub(pat, " ", vstr)
-            import Globals # for data
-            if Globals.DevelopmentMode:
+            if getConfiguration().debug_mode:
                 from traceback import format_exception
                 value = '\n' + ''.join(format_exception(t, vstr, tb))
             else:
                 value = '%s - %s' % (t, vstr)
-                
             if isinstance(v, Fault):
-                f=v
+                f = v
             elif isinstance(v, Exception):
-                f=Fault(-1, 'Unexpected Zope exception: %s' % value)
+                f = Fault(-1, 'Unexpected Zope exception: %s' % value)
             else:
-                f=Fault(-2, 'Unexpected Zope error value: %s' % value)
+                f = Fault(-2, 'Unexpected Zope error value: %s' % value)
         except ConflictError:
             raise
-        except:
-            f=Fault(-3, "Unknown Zope fault type")
+        except Exception:
+            f = Fault(-3, "Unknown Zope fault type")
 
         # Do the damage.
         self.setBody(f)
@@ -193,4 +216,5 @@ class Response:
 
         return tb
 
-response=Response
+
+response = Response
